<?php

use FinSearchAPI\Components\Findologic\Helper;

/**
 * Shopware 5.0
 * Copyright © 2015
 *
 * @category   Shopware_Plugins
 * @package    FinSearchAPI
 * @subpackage Bootstrap.php
 * @copyright  Copyright (c) 2015
 * @version    5.02.32.1
 * @author     FINDOLOGIC GmbH
 */

require_once 'Helpers/FinSearchAPIHelper.php';

/**
 * Class Shopware_Plugins_Frontend_FinSearchAPI_Bootstrap
 *
 * Standard bootstrap class for the plugin
 */
class Shopware_Plugins_Frontend_FinSearchAPI_Bootstrap extends Shopware_Components_Plugin_Bootstrap {

	/**
     * @var string Plugin version
     */

    const LABEL = 'FINDOLOGIC-Suche';
    const NAME = 'FinSearchAPI';

	/* To Add */
    private $shopKey;

    /**
     * @var int
     */
    private $sw;

    /**
     * @var helper
     */
    private $helper;

    /**
     * Initialise bootstrap object and set shopware version
     * @param $name
     * @param null $info
     */
    public function __construct($name, $info = null)
    {
        parent::__construct($name, $info);
        $this->sw = (int)Shopware()->Config()->version;
    }

    /**
     * Gets version tag of this plugin to display in manager
     *
     * @return string
     */
    public function getVersion() {
        return '5.02.32.1';
    }

    /**
     * The afterInit function registers the custom plugin models and namespaces.
     */
    public function afterInit()
    {
        $ds = DIRECTORY_SEPARATOR;
        $this->registerCustomModels();
        $this->Application()->Loader()->registerNamespace('FinSearchAPI', $this->Path());
        $this->Application()->Loader()->registerNamespace('FinSearchAPI\Components\Findologic', $this->Path() . $ds . 'Components' . $ds . 'Findologic');
        $this->helper = new Helper($this);
    }
    /* To Add end*/

    /**
     * Register plugin namespaces
     *
     * @return void
     */
    public function registerNamespace() {
        static $done = false;

        if (!$done) {
            $done = true;

            Shopware()->Loader()->registerNamespace('Shopware', dirname(__FILE__) . '/');
        }
    }

    /**
     * Plugin install method
     *
     * @return bool Installation success
     * @throws Enlight_Exception
     * @throws Exception
     */
    public function install() {
        self::registerNamespace();

        $this->checkLicense();

        if (!$this->assertVersionGreaterThen("5.0.0")) {
            throw new Enlight_Exception("This Plugin needs min shopware 5.0.0");
        }

        if (!function_exists('curl_init')) {
            throw new Enlight_Exception("This plugin requires the following php module to be active: cURL");
        }

        $this->createSnippets();
        $this->createForm();
        $this->createTranslations();
        $this->subscribeEvents();

        Shopware()->Db()->exec('CREATE TABLE IF NOT EXISTS fin_search_api_product_streams
          (id int NOT NULL AUTO_INCREMENT PRIMARY KEY,
           stream_id int NOT NULL,
           article_id int NOT NULL
          )');

        return array('success' => true, 'invalidateCache' => array('frontend', 'backend'));
    }

    /**
     * Plugin update method
     *
     * @param string $oldVersion Old version value
     * @return bool
     */
    public function update($oldVersion) {
        switch ($oldVersion) {
            case '4.0.1':
            case '4.01.03':
            case '4.01.04':
            case '4.01.05':
            case '4.01.06':
            case '4.01.07':
            case '4.01.08':
                $form = $this->Form();

                $form->setElement(
                    'text',
                    'exportpath',
                    array(
                        'label' => 'Export-Unterverzeichnis (kein führender ”/” [slash]',
                        'required' => false,
                        'value' => '',
                        'scope' => Shopware_Components_Form::SCOPE_SHOP
                    )
                );

                $form->save();

                $translations = array(
                    'en_GB' => array(
                        'exportpath' => 'export subdirectory (no leadin "/" [slash])'
                    )
                );

                $shopRepository = Shopware()->Models()->getRepository('\Shopware\Models\Shop\Locale');

                foreach ($translations as $locale => $snippets) {
                    $localeModel = $shopRepository->findOneBy(array('locale' => $locale));

                    foreach ($snippets as $element => $snippet) {
                        if ($localeModel === null) {
                            continue;
                        }

                        $elementModel = $form->getElement($element);

                        if ($elementModel === null) {
                            continue;
                        }

                        $translationModel = new \Shopware\Models\Config\ElementTranslation();
                        $translationModel->setLabel($snippet);
                        $translationModel->setLocale($localeModel);
                        $elementModel->addTranslation($translationModel);
                    }
                }
                break;
            case '4.01.09':
            case '4.01.10':
            case '4.01.11':
            case '4.01.12':
                $form = $this->Form();

                $form->setElement(
                    'combo',
                    'exportdate',
                    $this->createDateDropdown(
                        'folgendes Datum für Artikelsortierung nutzen',
                        Shopware_Components_Form::SCOPE_SHOP
                ));

                $form->save();

                $translations = array(
                    'en_GB' => array(
                        'exportdate' => 'use the following date for article sorting'
                    )
                );

                $shopRepository = Shopware()->Models()->getRepository('\Shopware\Models\Shop\Locale');

                foreach ($translations as $locale => $snippets) {
                    $localeModel = $shopRepository->findOneBy(array('locale' => $locale));

                    foreach ($snippets as $element => $snippet) {
                        if ($localeModel === null) {
                            continue;
                        }

                        $elementModel = $form->getElement($element);

                        if ($elementModel === null) {
                            continue;
                        }

                        $translationModel = new \Shopware\Models\Config\ElementTranslation();
                        $translationModel->setLabel($snippet);
                        $translationModel->setLocale($localeModel);
                        $elementModel->addTranslation($translationModel);
                    }
                }
                break;
            case '4.01.13':
                $form = $this->Form();

                $form->setElement(
                    'boolean',
                    'pluginstatus',
                    array(
                        'label' => 'Plugin für diesen Shop aktivieren?',
                        'required' => true,
                        'value' => false,
                        'scope' => Shopware_Components_Form::SCOPE_SHOP
                    )
                );

                $form->save();

                $translations = array(
                    'en_GB' => array(
                        'pluginstatus' => 'activate plugin for this shop?'
                    )
                );

                $shopRepository = Shopware()->Models()->getRepository('\Shopware\Models\Shop\Locale');

                foreach ($translations as $locale => $snippets) {
                    $localeModel = $shopRepository->findOneBy(array('locale' => $locale));

                    foreach ($snippets as $element => $snippet) {
                        if ($localeModel === null) {
                            continue;
                        }

                        $elementModel = $form->getElement($element);

                        if ($elementModel === null) {
                            continue;
                        }

                        $translationModel = new \Shopware\Models\Config\ElementTranslation();
                        $translationModel->setLabel($snippet);
                        $translationModel->setLocale($localeModel);
                        $elementModel->addTranslation($translationModel);
                    }
                }
                break;
            case '4.01.14':
            case '4.01.15':
            case '4.01.16':
                $form = $this->Form();

                $form->setElement(
                    'text',
                    'thumbnailsize',
                    array(
                        'label' => 'ID der Thumbnail-Größe',
                        'required' => false,
                        'value' => '',
                        'scope' => Shopware_Components_Form::SCOPE_SHOP
                    )
                );

                $form->save();

                $translations = array(
                    'en_GB' => array(
                        'thumbnailsize' => 'ID of thumbnail-size'
                    )
                );

                $shopRepository = Shopware()->Models()->getRepository('\Shopware\Models\Shop\Locale');

                foreach ($translations as $locale => $snippets) {
                    $localeModel = $shopRepository->findOneBy(array('locale' => $locale));

                    foreach ($snippets as $element => $snippet) {
                        if ($localeModel === null) {
                            continue;
                        }

                        $elementModel = $form->getElement($element);

                        if ($elementModel === null) {
                            continue;
                        }

                        $translationModel = new \Shopware\Models\Config\ElementTranslation();
                        $translationModel->setLabel($snippet);
                        $translationModel->setLocale($localeModel);
                        $elementModel->addTranslation($translationModel);
                    }
                }
                break;

            case '4.01.17':
            case '4.01.18':
            case '4.01.19':
            case '4.01.20':
            case '4.01.21':
            case '4.01.22':
            case '4.01.23':
            case '4.01.24':
            case '4.01.25':
            case '4.01.26':
                $sql = "DELETE FROM s_core_snippets WHERE namespace LIKE '%FinSearchAPI%';";
                Shopware()->Db()->exec($sql);

                $this->createSnippets();
                break;
            case '4.01.27':
                $sql = "UPDATE s_core_snippets
                        SET `namespace` = 'frontend/plugins/fin_search_api/search'
                        WHERE namespace LIKE '%FinSearchAPI%';";
                Shopware()->Db()->exec($sql);
                break;
            case '5.01.00':
            case '5.01.01':
            case '5.01.02':
            case '5.01.03':
            case '5.01.04':
            case '5.02.01':
            case '5.02.02':
            case '5.02.03':
            case '5.02.04':
            case '5.02.05':
            case '5.02.06':
            case '5.02.07':
            case '5.02.08':
            case '5.02.09':
            case '5.02.10':
            case '5.02.11':
            case '5.02.12':
            case '5.02.13':
            case '5.02.14':
            case '5.02.15':
            case '5.02.16':
            case '5.02.17':
            case '5.02.18':
            case '5.02.19':
            case '5.02.20':
            case '5.02.21':
            case '5.02.22':
            case '5.02.23':
            case '5.02.24':
            case '5.02.25':
            case '5.02.26':
            case '5.02.27':
            case '5.02.28':
            case '5.02.29':
            case '5.02.30':
            case '5.02.31':
                return true;
            default:
                return true;
        }
    }

    /**
     * Plugin uninstall method
     *
     * @return bool Success
     */
    public function uninstall() {
        Shopware()->Db()->exec('
            DELETE FROM s_core_snippets
            WHERE `name` LIKE "FinSearchAPI%" OR `namespace` LIKE "%FinSearchAPI%";
        ');

        Shopware()->Db()->exec('DROP TABLE IF EXISTS fin_search_api_product_streams');

        return true;
    }

    /**
     * Activates the plugin
     *
     * @return boolean Success
     */
    public function enable() {
        $this->checkLicense();

        return true;
    }

    /**
     * Deactivates the plugin
     *
     * @return boolean Success
     */
    public function disable() {
        return true;
    }

    /**
     * Gets the name for the plugin
     *
     * @return string Plugin name
     */
    public function getLabel() {
        return self::LABEL;
    }

    /**
     * Gets the main info for the plugin
     *
     * @return mixed
     */
    public function getInfo() {
        return include( dirname(__FILE__) . '/Meta.php' );
    }

    /**
     * Creates plugin config form
     *
     * @return void
     */
    protected function createForm() {
        $form = $this->Form();

        $form->setElement(
            'boolean',
            'pluginstatus',
            array(
                'label' => 'Plugin für diesen Shop aktivieren?',
                'required' => true,
                'value' => false,
                'scope' => Shopware_Components_Form::SCOPE_SHOP
            )
        );
        $form->setElement(
            'text',
            'shopkey',
            array(
                'label' => 'Shopkey',
                'required' => true,
                'value' => '',
                'scope' => Shopware_Components_Form::SCOPE_SHOP
            )
        );
        $form->setElement(
            'text',
            'serviceurl',
            array(
                'label' => 'FINDOLOGIC/Service URL',
                'required' => true,
                'value' => '',
                'scope' => Shopware_Components_Form::SCOPE_SHOP
            )
        );
        $form->setElement(
            'text',
            'exportlimit',
            array(
                'label' => 'Export-Limit pro Step',
                'required' => true,
                'value' => '100',
                'scope' => Shopware_Components_Form::SCOPE_SHOP
            )
        );
        $form->setElement(
            'text',
            'pagearticlescount',
            array(
                'label' => 'Anzahl der Ergebnisse pro Seite (Mehrere Einträge durch Semikolon (;) trennen)',
                'required' => true,
                'value' => '12;24;36;48',
                'scope' => Shopware_Components_Form::SCOPE_SHOP
            )
        );
        $form->setElement(
            'boolean',
            'categorynavigationtop',
            array(
                'label' => 'Kategorie anzeigen Navigation auf der Oberseite?',
                'required' => true,
                'value' => true,
                'scope' => Shopware_Components_Form::SCOPE_SHOP
            )
        );
        $form->setElement(
            'boolean',
            'findologiconcategorypage',
            array(
                'label' => 'FINDOLOGIC ist auf Kategorieseite?',
                'required' => true,
                'value' => true,
                'scope' => Shopware_Components_Form::SCOPE_SHOP
            )
        );
        $form->setElement(
            'boolean',
            'displaybranding',
            array(
                'label' => 'Branding anzeigen?',
                'required' => true,
                'value' => true,
                'scope' => Shopware_Components_Form::SCOPE_SHOP
            )
        );
        $form->setElement(
            'combo',
            'findologiclogo',
            $this->createLogoDropDown(
                'Suchergebnis-Logo',
                Shopware_Components_Form::SCOPE_SHOP
            )
        );
        $form->setElement(
            'text',
            'exportpath',
            array(
                'label' => 'Export-Unterverzeichnis (kein führender ”/” [slash]',
                'required' => false,
                'value' => '',
                'scope' => Shopware_Components_Form::SCOPE_SHOP
            )
        );
        $form->setElement(
            'combo',
            'exportdate',
            $this->createDateDropdown(
                'folgendes Datum für Artikelsortierung nutzen',
                Shopware_Components_Form::SCOPE_SHOP
            )
        );
        $form->setElement(
            'text',
            'thumbnailsize',
            array(
                'label' => 'ID der Thumbnail-Größe',
                'required' => false,
                'value' => '',
                'scope' => Shopware_Components_Form::SCOPE_SHOP
            )
        );

        $form->save();
    }

    /**
     * Creates logo dropdown
     *
     * @param string $label Label
     * @param string $scope Scope
     * @return array|bool Image array, null if error occurred
     */
    protected function createLogoDropDown($label, $scope) {
        $data = array();

        $dir = dirname(__FILE__) . '/Views/frontend/_Resources/Logos/';

        if (!is_dir($dir)) {
            return false;
        }

        $d = dir($dir);

        while ($name = $d->read()) {
            if (!preg_match('/\.(png|gif|jpg|jpeg)$/', $name)) {
                continue;
            }

            $data[] = array($name, $name);
        } $d->close();

        return array(
            'label' => $label,
            'required' => true,
            'value' => 'original_transparent.png',
            'scope' => $scope,
            'store' => $data
        );
    }

    /**
     * Creates plugin config dropdown for exported date
     *
     * @param string $label Label
     * @param string $scope Scope
     * @return array
     */
    protected function createDateDropdown($label, $scope) {
        return array(
            'label' => $label,
            'required' => true,
            'value' => 0,
            'scope' => $scope,
            'store' => array(
                array(0, 'Einstelldatum'),
                array(1, 'Erscheinungsdatum')
            )
        );
    }

    /**
     * Creates plugin config form translations
     *
     * @return void
     */
    protected function createTranslations() {
        $form = $this->Form();

        $translations = array(
            'en_GB' => array(
                'pluginstatus' => 'Activate Plugin for this Shop?',
                'shopkey' => 'Shopkey',
                'serviceurl' => 'FINDOLOGIC/Service URL',
                'exportlimit' => 'Number of records per export step',
                'pagearticlescount' => 'Number of Articles per Page (seperate multiple items with Semicolon (;))',
                'categorynavigationtop' => 'Display category navigation on top',
                'findologiconcategorypage' => 'FINDOLOGIC is present on category page',
                'displaybranding' => 'Display branding?',
                'findologiclogo' => 'Search result Logo',
                'exportdate' => 'Use the following date for article sorting',
                'thumbnailsize' => 'ID of thumbnail-size',
                'exportpath' => 'Export subfolder (without leading slash "/")'
            )
        );

        $shopRepository = Shopware()->Models()->getRepository('\Shopware\Models\Shop\Locale');

        foreach ($translations as $locale => $snippets) {
            $localeModel = $shopRepository->findOneBy(array('locale' => $locale));

            foreach ($snippets as $element => $snippet) {
                if ($localeModel === null) {
                    continue;
                }

                $elementModel = $form->getElement($element);

                if ($elementModel === null) {
                    continue;
                }

                $translationModel = new \Shopware\Models\Config\ElementTranslation();
                $translationModel->setLabel($snippet);
                $translationModel->setLocale($localeModel);
                $elementModel->addTranslation($translationModel);
            }
        }
    }

    /**
     * Creates Events/Hooks for the plugin
     *
     * @return void
     */
    protected function subscribeEvents() {
        $this->subscribeEvent(
            'Enlight_Controller_Dispatcher_ControllerPath_Frontend_FinSearchAPI',
            'onGetControllerPathFrontend'
        );
        $this->subscribeEvent(
            'Enlight_Controller_Action_PostDispatch',
            'onPostDispatch'
        );
        $this->subscribeEvent(
            'Shopware_Controllers_Frontend_Search::indexAction::before',
            'before_searchIndexAction'
        );

        /* Subscribe new events */

        $this->subscribeEvent(
            'Enlight_Controller_Action_PostDispatch_Frontend', 'onPostDispatchFrontend'
        );
        $this->subscribeEvent(
            'Enlight_Controller_Action_PostDispatch_Frontend_Listing', 'onPostDispatchListing'
        );
        $this->subscribeEvent(
            'Enlight_Controller_Action_PostDispatch_Frontend_Search', 'onPostDispatchSearch'
        );
        $this->subscribeEvent(
            'Shopware_Controllers_Backend_Config_Before_Save_Config_Element', 'onConfigSaveForm'
        );
        $this->subscribeEvent(
            'Enlight_Controller_Action_PostDispatch_Frontend_Detail', 'onPostDispatchDetail'
        );
        $this->subscribeEvent(
            'Enlight_Controller_Action_Frontend_Checkout_Finish', 'onCheckoutFinish'
        );
        $this->subscribeEvent(
            'Enlight_Controller_Action_Frontend_Checkout_Cart', 'onCheckoutConfirm'
        );
        $this->subscribeEvent(
            'Enlight_Controller_Action_Frontend_Checkout_Confirm', 'onCheckoutConfirm'
        );
        $this->subscribeEvent(
            'Enlight_Controller_Action_Frontend_Checkout_addArticle', 'onCheckoutConfirm'
        );
        $this->subscribeEvent(
            'Enlight_Controller_Action_Frontend_Checkout_ajaxAddArticle', 'onCheckoutConfirm'
        );
    }

    /* Start Events */

    /**
     * Validated that each shop has its own shop key. Trims shop key value.
     * @param Enlight_Event_EventArgs $args
     * @return array
     * @throws Exception
     */
    public function onConfigSaveForm(Enlight_Event_EventArgs $args)
    {
        $params = $args->getSubject()->Request()->getParams();
        if ($params['name'] !== 'FinSearchAPI') {
            return null;
        }

        $values = $args->getReturn();
        $keys = array();

        $numberOfShops = $this->getNumberOfShops();
        /**
         * @var int $shopId
         * @var Shopware\Models\Config\Value $value
         */
        foreach($values as $shopId => &$value) {
            $value->setValue(trim($value->getValue()));
            $val = $value->getValue();
            if (!$val) {
                continue;
            }

            if ($numberOfShops == 1) {
                if (isset($keys[$val])) {
                    throw new \Exception('Each shop must have its own shop key!');
                }
            }

            $keys[$val] = 1;
        }

        return $values;
    }

    /**
     * Event responsible for category pages
     *
     * @param Enlight_Event_EventArgs $args
     * @throws Exception
     */
    public function onPostDispatchListing(Enlight_Event_EventArgs $args)
    {

        $config = Shopware()->Plugins()->Frontend()->FinSearchAPI()->Config();

        if (!$config['pluginstatus']) {
            return;
        }

        try {
            $controller = $args->getSubject();
            $request = $controller->Request();
            $shopUrl = Shopware()->Shop()->getBasePath() . $request->getPathInfo() . '?';
            $requestUrl = $controller->Request()->getRequestUri();
            $helper = new FinSearchAPIHelper();

            if (strpos($requestUrl, 'findologic=off') == false && $helper->aliveTest()) {
                $serviceBasicUrl = $config['serviceurl'];
                $view = ($request->getParam('view')) ? $request->getParam('view') : 1;
                $data = new stdClass();
                $url = '';

                $keywords = urlencode($request->getParam('sSearch'));
                $order = $helper->getOrderParam($request);
                $count = $helper->getCountParam($request);
                $params = '';
                $attrib = $request->getParams()['attrib'];
                $shopKey = Shopware()->Plugins()->Frontend()->FinSearchAPI()->Config()['shopkey'];

                $categoryName = $helper->buildCategoryName($request);
                $template_file = '';

                if (!empty($attrib)) {
                    // category page WITH filters selected
                    $attrib['order'] = $order;
                    $attrib['count'] = $count;

                    $xml = $helper->callFindologicForCategoryPage(
                        $serviceBasicUrl,
                        $shopKey,
                        $categoryName,
                        $attrib,
                        $count,
                        $order,
                        $view
                    );
                    $helper->setFilter($request->getParams(), $url, $data, $params);
                    $helper->buildCategoryUrl($url, $order, $count, $attrib);
                    $helper->buildPagesizeDropdown($data, $count);
                    $helper->buildFilters($xml, $data);
                    $helper->buildProductData($xml, $data);
                    $helper->buildOrderDropdown($data, $order);
                    $helper->buildPaging($xml, $keywords, $params, $view, $data, $shopUrl);

                    // If we are displaying navigation on top or left
                    if ($config['categorynavigationtop']) {
                        $template_file = 'plugins/fin_search_api/fin_category.tpl';
                    } else {
                        $template_file = 'plugins/fin_search_api/fin_category_left.tpl';
                    }

                } else {
                    // category page with NO filters selected
                    $attrib['order'] = $order;
                    $attrib['count'] = $count;

                    $category_service = $serviceBasicUrl . 'selector.php?shopkey=' .
                        $shopKey . '&selected[cat][]=' . $categoryName . '&count=0';

                    $ctx = stream_context_create(array('http'=>
                        array('timeout' => 3)
                    ));

                    $string = file_get_contents($category_service, false, $ctx);

                    if ($string) {
                        $xml = new SimpleXMLElement($string);
                        $helper->buildFilters($xml, $data);
                        $helper->buildOrderDropdown($data, $order);
                    }

                    // If we are displaying navigation on top or left
                    if ($config['categorynavigationtop']) {
                        $template_file = 'plugins/fin_search_api/fin_category_initial.tpl';
                    } else {
                        $template_file = 'plugins/fin_search_api/fin_category_initial_left.tpl';
                    }
                }

                $exportHelper = new \FinSearchAPI\Helpers\ExportHelper();
                $filters = $data->filter['Color']['filter'];

                foreach($filters as &$filterValue) {
                    $media = $exportHelper->getMedia($filterValue['name']);
                    $filterValue['image_path'] = $media[0]['media']['path'];
                }
                $data->filter['Color']['filter'] = $filters;

                $tpl_data = array(
                    'request_url' => $url,
                    'shopurl' => Shopware()->Shop()->getBasePath().$shopUrl,
                    'order_dropdown' => $data->orderDropdown,
                    'pagesize_dropdown' => $data->pagesizeDropdown,
                    'selected_filter' => $data->selected_filter,
                    'filter' => $data->filter,
                    'disable_cat_filter' => true,
                    'products' => $data->products,
                    'url_params' => $params,
                    'paging' => $data->paging,
                    'navigation_is_left' => !$config['categorynavigationtop'],
                    'displayBranding' => $config['displaybranding'],
                );

                if ((!empty($xml) || !empty($string)) && $config['findologiconcategorypage']) {
                    $view = $controller->View();
                    // __debug(get_class_methods(get_class($view)));die;
                    $view->extendsTemplate($template_file);
                    $view->findologic = $tpl_data;
                }
            }
        } catch (Exception $ex) {
            // __debug($ex->getTraceAsString());die; // For testing purposes
            throw $ex;
        }
    }

    /* End Events */

    /**
     * @param $xml
     * @return array|string
     */
    private function simpleXML2Array($xml)
    {
        if (is_string($xml) || is_numeric($xml)) {
            return $xml;
        }

        $array = (array)$xml;
        if (count($array) == 0) {
            $array = (string)$xml;
        }

        if (is_array($array)) {
            foreach ($array as $key => $value) {
                if (is_object($value)) {
                    if ($value instanceof \SimpleXMLElement) {
                        $array[$key] = $this->simpleXML2Array($value);
                    }
                } else {
                    $array[$key] = $this->simpleXML2Array($value);
                }
            }
        }

        return $array;
    }

    /**
     * Get shop key
     * 
     * @return bool|string
     */
    private function getShopKey()
    {
        if (!$this->shopKey) {

            $shopName = Shopware()->Shop()->getName();
            $config = Shopware()->Plugins()->Frontend()->FinSearchAPI()->Config();
            $shop = Shopware()->Models()->getRepository('Shopware\Models\Shop\Shop')
                ->findOneBy(array('name' => $shopName));
            $shopId = $shop->getId();
            $configShop = Shopware()->Models()->getRepository('Shopware\Models\Config\Value')
                ->findOneBy(array('value' => $config->get('shopkey')));

            if ($configShop && $configShop->getShop()->getId() == $shopId) {
                $this->shopKey = trim($config->get('shopkey'));
            } else {
                $this->shopKey = false;
            }
        }

        return $this->shopKey;
    }

    /**
     * Do we use FINDOLOGIC
     * 
     * @param Enlight_Event_EventArgs $args
     * @return bool
     */
    private function useFindologic(Enlight_Event_EventArgs $args)
    {

        if (!$this->getShopKey()) {
            return false;
        }

        $requestUrl = $args->getSubject()->Request()->getRequestUri();
        return strpos($requestUrl, 'findologic=off') === false;
    }

    /**
     * @param Enlight_Event_EventArgs $args
     * @return string
     * @throws Exception
     */
    public function onGetControllerPathFrontend(Enlight_Event_EventArgs $args) {
        self::registerNamespace();

        $this->checkLicense();
        return dirname(__FILE__) . '/Controllers/Frontend/FinSearchAPI.php';
    }

    /**
     * Mainly sets templatedir and adds js and css to html head
     *
     * @param Enlight_Event_EventArgs $args Request arguments
     * @throws Exception
     */
    public function onPostDispatch(Enlight_Event_EventArgs $args) {
        self::registerNamespace();

        $this->checkLicense();

        $response = $args->getSubject()->Response();
        $request = $args->getSubject()->Request();
        $view = $args->getSubject()->View();

        if ($response->isException() || $request->getModuleName() != 'frontend') {
            return;
        }

        if (!Shopware()->Plugins()->Frontend()->FinSearchAPI()->Config()->pluginstatus) {
            return;
        }

        $config = Shopware()->Plugins()->Frontend()->FinSearchAPI()->Config();

        $view->addTemplateDir($this->Path() . "Views/frontend/");

        if (!empty($config->shopkey)) {
            $myValues = array(
                'logo' => '_Resources/Logos/' . $config->findologiclogo,
                'hashedShopkey' => strtoupper(md5($this->cleanShopkey($config->shopkey))),
            );

            $view->finSearchApi = $myValues;
            $view->extendsTemplate("plugins/fin_search_api/includeAll.tpl");
        } else {
            $view->extendsTemplate("plugins/fin_search_api/includeSearchOnly.tpl");
        }
    }

    /**
     * Redirects search to the findologic search, if findologic available
     *
     * @param Enlight_Hook_HookArgs $args Request arguments
     * @throws Exception
     */
    public function before_searchIndexAction(Enlight_Hook_HookArgs $args) {
        self::registerNamespace();

        $this->checkLicense();

        if (!Shopware()->Plugins()->Frontend()->FinSearchAPI()->Config()->pluginstatus) {
            return;
        }

        $request = $args->getSubject()->Request();

        $config = Shopware()->Plugins()->Frontend()->FinSearchAPI()->Config();

        $responseAliveTest = '';

        if ($config->shopkey != '' && $config->serviceurl != '') {
            $googleBots = array(
                'Mozilla/5.0 (compatible; Googlebot/2.1; +http://www.google.com/bot.html)',
                'Googlebot/2.1 (+http://www.googlebot.com/bot.html)',
                'Googlebot/2.1 (+http://www.google.com/bot.html)',
            );

            if (!in_array($_SERVER['HTTP_USER_AGENT'], $googleBots)) {
                if (!$request->getParam('fallback')) {
                    $helper = new FinSearchAPIHelper();
                    $responseAliveTest = $helper->aliveTest();

                    if ($responseAliveTest) {

                        $params = $request->getParams();
                        $sSearch = $request->getParam('sSearch');
                        unset($params['sSearch']);
                        $params['controller'] = 'FinSearchAPI';
                        $params['action'] = 'search';
                        //__debug($params);die;

                        $addParamString = '';

                        if ($params['attrib'] && count($params['attrib']) > 0) {
                            foreach ($params['attrib'] as $key => $value) {
                                if ($key == 'cat') {
                                    $addParamString .= '&catFilter=' . urlencode($value[0]);
                                } else {
                                    $addParamString .= '&attrib[' . urlencode($key) . '][]=' . urlencode($value[0]);
                                }
                            }
                        }

                        unset($params['attrib']);
                        $redirectURL = Shopware()->Front()->Router()->assemble($params)
                            . '?sSearch=' . urlencode($sSearch) . $addParamString;
                        $args->getSubject()->redirect($redirectURL);
                    }
                }
            }
        }
    }

    /**
     * Cleans shopkey only a-z,A-Z,0-9
     *
     * @param string $key Shop key
     * @return mixed
     */
    private function cleanShopkey($key) {
        $key = preg_replace("/[^a-zA-Z0-9]/i", "", $key);

        return $key;
    }

    /**
     * Cleans serviceurl only a-z,A-Z,0-9,/,:,.
     *
     * @param string $url Service URL
     * @return string Clear URL
     */
    private function cleanServiceurl($url) {
        $url = preg_replace("/[^a-zA-Z0-9\/:._-]/i", "", $url);

        return $url;
    }

    /**
     * Creates snippets for the frontend
     *
     * @return void
     */
    protected function createSnippets() {
        $sql = "SELECT DISTINCT s.shopID as id, l.locale, CONCAT(IF(o.id=1, 'Default', o.name), ' / ', l.language) as name
                FROM s_core_snippets s, s_core_locales l, s_core_shops o
                WHERE l.id = s.localeID
                AND o.id = s.shopID";

        $result = Shopware()->Db()->fetchAll($sql);

        $snippets =$this->getSnippetsFromFile();

        foreach ($result as $ShopLocale) {
            $localeID = Shopware()->Db()->fetchOne(
                'SELECT id FROM s_core_locales
                 WHERE locale="' . $ShopLocale['locale'] . '"'
            );
            $sql = 'REPLACE INTO s_core_snippets (namespace, name, localeID, shopID, value, created, updated)
                    VALUES ("frontend/plugins/fin_search_api/search", ?, "'
                    . $localeID . '", "' . $ShopLocale['id'] . '", ?, NOW(), NOW())';

            $locales = array('de_DE', 'de_CH', 'de_AT');
            if (in_array($ShopLocale['locale'], $locales)) {
                $ShopLocale['locale'] = 'de_DE';
            }

            foreach ($snippets as $name => $snippet) {
                if (!array_key_exists($ShopLocale['locale'], $snippet)) {
                    $ShopLocale['locale'] = 'other';
                }

                $insert = array($name, $snippet[$ShopLocale['locale']]);
                Shopware()->Db()->query($sql, $insert);
            }
        }
    }

    /**
     * Gets snippet from snippets.json file
     *
     * @return array|mixed JSON decoded snippet
     */
    private function getSnippetsFromFile() {
        $file = __DIR__ . '/snippets.json';
        $snippets = array();

        if (file_exists($file)) {
            $snippets = json_decode(file_get_contents($file), true);
        }

        return $snippets;
    }

    /**
     * Returns total number of shops
     *
     * @return int
     */
    private function getNumberOfShops()
    {
        $shopRepository = Shopware()->Models()->getRepository('\Shopware\Models\Shop\Shop');
        $numberOfShops =  $shopRepository->findAll();

        return count($numberOfShops);
    }

    /**
     * CheckLicense()-method for FinSearchAPI
     *
     * @param bool $throwException If it will throw exception
     * @return bool|string|void
     * @throws Exception
     */
    public function checkLicense($throwException = true) {
        return true;
        try {
            static $r, $module = 'FinSearchAPI';
            if (!isset($r)) {
                $s = base64_decode('RcTCQCxatQnEAPR20R+1IsN30Zw=');
                $c = base64_decode('DieXpYKWyyhZizQHhF8z6j6z61A=');
                $r = sha1(uniqid('', true), true);
                /** @var $l Shopware_Components_License */
                $l = $this->Application()->License();
                $i = $l->getLicense($module, $r);
                $t = $l->getCoreLicense();
                $u = strlen($t) === 20 ? sha1($t . $s . $t, true) : 0;
                $r = $i === sha1($c . $u . $r, true);
            }
            if (!$r && $throwException) {
                throw new Exception('License check for module "' . $module . '" has failed.');
            }
            return $r;
        } catch (Exception $e) {
            if ($throwException) {
                throw new Exception('License check for module "' . $module . '" has failed.');
            } else {
                return false;
            }
        }
    }
}

if (!function_exists('__debug')) {

    function __debug($msg) {
        echo "<pre>";
        print_r($msg);
        echo "</pre>";
        echo "<hr />";
    }

}
