<?php

namespace FinSearchAPI\Components\Findologic;

class Helper
{

    /**
     * Get path for supplied category.
     *
     * @param int $catId
     * @return array[string]
     */
    public function getCategories($catId)
    {
        $category = Shopware()->Modules()->Categories()->sGetCategoriesByParent($catId);
        $articleCategories = array();

        foreach ($category as $cat) {
            $articleCategories[] = $cat['name'];
        }

        if (count($articleCategories) > 5) {
            $articleCat = array_slice($articleCategories, 0, 5);
        } else {
            $articleCat = $articleCategories;
        }

        return array_reverse($articleCat);
    }

    /**
     * Get path for main category of supplied article.
     *
     * @param int $articleId
     * @return array[string]
     */
    public function getCartCategories($articleId)
    {
        $sql = "SELECT categoryID FROM s_articles_categories where articleID =?";
        $categoryID = Shopware()->Db()->fetchOne($sql, array($articleId));

        return $this->getCategories($categoryID);
    }

    /**
     * Helper function, gets string between defined start and end string parts
     */
    public function getStringBetween($string, $start, $end)
    {
        $string = ' ' . $string;
        $ini = strpos($string, $start);
        if ($ini == 0) {
            return '';
        }

        $ini += strlen($start);
        $len = strpos($string, $end, $ini) - $ini;

        return substr($string, $ini, $len);
    }

    /**
     * Get product delivery time
     *
     * @param int $product_id Product ID
     * @return int
     */
    public function getDeliveryTime($product_id)
    {
        $result = 0;
        try {
            $builder = Shopware()->Models()->createQueryBuilder();
            $builder->select('details.shippingTime')
                ->from('Shopware\Models\Article\Detail', 'details')
                ->where('details.articleId = ?1')
                ->setParameter(1, (int)$product_id);
            $query = $builder->getQuery();
            $result = $query->getOneOrNullResult(\Doctrine\ORM\AbstractQuery::HYDRATE_ARRAY);
            $result = empty($result['1']) ? 'on stock' : ($result['1'] . ' day(s)');

        }catch (Exception $ex) {
            die($ex->getMessage());
        }

        return $result;
    }

}
