<?php

namespace FinSearchAPI\Components\XmlParser;


use SimpleXMLElement;

class AttributeCollection implements \IteratorAggregate, \Countable
{
    private $attributes = array();

    public function __construct(SimpleXMLElement $element)
    {
        foreach ($element->attributes() as $key => $value) {
            $this->attributes[$key] = (string)$value;
        }
    }

    public function __toString()
    {
        return implode(", ", $this->attributes);
    }

    /**
     * @param $key
     * @return bool
     */
    public function hasAttribute($key)
    {
        return isset($this->attributes[$key]);
    }

    /**
     * @param $key
     * @return string|null
     */
    public function getAttributeOrNull($key)
    {
        return $this->hasAttribute($key) ? $this->attributes[$key] : null;
    }

    public function getIterator()
    {
        return new \ArrayIterator($this->attributes);
    }

    /**
     * @return array
     */
    public function toArray()
    {
        return $this->attributes;
    }

    /**
     * @return int
     */
    public function count()
    {
        return count($this->attributes);
    }

    public function __debugInfo()
    {
        return $this->attributes;
    }
}
