<?php

namespace FinSearchAPI\Components\XmlParser;


class Field
{
    public $name;
    public $value;
    private $element;

    /**
     * @var AttributeCollection
     */
    public $attributes;

    public function __construct(\SimpleXMLElement $element)
    {
        $this->name = $element->getName();
        $this->element = $element;
        $this->value = (string)$element;
        $this->attributes = new AttributeCollection($element);
    }

    public function __toString()
    {
        return $this->toString();
    }

    public function toStringOrNull()
    {
        return (strlen($this->value) > 0) ? $this->toString() : null;
    }

    public function toString()
    {
        return trim((string)$this->value);
    }

    public function toFloat()
    {
        return floatval($this->toString());
    }

    public function toInteger()
    {
        return (int)$this->toString();
    }

    public function toBool()
    {
        return filter_var($this->toString(), FILTER_VALIDATE_BOOLEAN);
    }

    /**
     * @param $key
     * @return null|string
     */
    public function getAttributeOrNull($key)
    {
        return $this->attributes->getAttributeOrNull($key);
    }

    public function __debugInfo()
    {
        $result = array(
            'name' => $this->name,
            'value' => $this->toString()
        );

        if (count($this->attributes) > 0) {
            $result = $result + array('attributes' => $this->attributes->toArray());
        }

        return $result;
    }
}
