<?php
/**
 * Plugin for Shopware
 *
 * @version $Id: FinSearchAPI.php 5166 2014-11-17 13:47:56Z r.siemoneit $
 * @copyright:   found in /lic/copyright.txt
 */
require_once 'engine/Shopware/Plugins/Community/Frontend/FinSearchAPI/Helpers/FinSearchAPIHelper.php';
require_once 'engine/Shopware/Plugins/Community/Frontend/FinSearchAPI/Helpers/ExportHelper.php';

/**
 * Class for FINDOLOGIC export, autocomplete query and search
 */
require_once 'engine/Shopware/Plugins/Community/Frontend/FinSearchAPI/Components/Findologic/Helper.php';
class Shopware_Controllers_Frontend_FinSearchAPI extends Enlight_Controller_Action
{
    ##########################
    ###    Export Start    ###
    ##########################

    /**
     * @var string Delimiter Tab
     */
    protected $delimiter = "\t";

    /**
     * @var string Double Quotes
     */
    protected $enclosure = '"';

    /**
     * @var string Language Id
     */
    private $language_id;

    /**
     * @var object Article Object
     */
    private $ar;

    /**
     * @var \FinSearchAPI\Helpers\ExportHelper
     */
    private $exportHelper;

    /**
     * @var string
     */
    private $languageCode;

    /* Navigation */

    public function ajaxFiltersAction()
    {
        $attributes = $this->Request()->get('attrib');
        $page = $this->Request()->get('page');
        $pageSize = $this->Request()->get('pageSize');

        $count = ($pageSize && is_numeric($pageSize) ? $pageSize : 12);
        $first = ($page && is_numeric($page) ? ($page - 1) * $pageSize : 0);
        $attributes = array('attrib' => $attributes);

        $category = $this->Request()->getParam('category');

        $shopKey = Shopware()->Plugins()->Frontend()->FinSearchAPI()->Config()['shopkey'];
        $attribs = str_replace(' ', '+', urldecode(http_build_query($attributes)));

        try {
            $string = file_get_contents('http://service.findologic.com/ps/xml_2.0/selector.php?shopkey=' . $shopKey . '&selected[cat][]=' . $category . '&' . $attribs . '&first=' . $first . '&count=' .$count);
        } catch  (Exception $e){
            throw new Exception();
        }

        $xml = new SimpleXMLElement($string);

        $graph = new searchResult($xml);
        $product_array = array();

        foreach ($graph->productsCollection->products as $product) {
            $product_array[] = $product->attributes->getAttributeOrNull('id');
        }

        $articleData = $this->prepareArticleData($product_array);

        $flQuery = $xml->query;
        $flResults = $xml->results->count;

        $filters = $xml->filters->filter;

        $elements = $filters->count();
        $counter =0;

        $result = array();

        for($counter; $counter < $elements; $counter++) {
            $result[$counter] = $this->xml2array($filters[$counter]);
        }

        $this->View()->assign('fin_filters', $result);
        $this->View()->assign('flArticles', $articleData);
        $this->View()->assign('flQuery', $flQuery);
        $this->View()->assign('flResults', $flResults);
        $this->View()->assign('pageSize', $pageSize);
        $this->View()->assign('page', $page);
        $this->View()->assign('division', ceil($flResults/$pageSize));

        $this->Front()->Plugins()->ViewRenderer()->setNoRender();
        $this->Response()->setBody(json_encode(array(
            'filters' => $this->View()->fetch(dirname(__FILE__) . "/../../Views/frontend/fin_search_api/ajax_filters.tpl"),
            'products' => $this->View()->fetch(dirname(__FILE__) . "/../../Views/frontend/fin_search_api/fin_listing.tpl")
        )));
    }

    private function prepareArticleData($articleIds) {
        $result = array();

        foreach ($articleIds as $articleId) {
            $article = Shopware()->Models()->getRepository('Shopware\Models\Article\Article')
                ->findOneBy(array(
                    'id' => $articleId
                ));
            $sArticle = Shopware()->Modules()->Articles()->sGetArticleById($articleId);

            $result[$articleId]['articleId'] = $articleId;
            $result[$articleId]['orderNumber'] = $sArticle['ordernumber'];
            $result[$articleId]['name'] = $article->getName();
            $result[$articleId]['description'] = $article->getDescription();
            $result[$articleId]['price'] = $sArticle['price'];
            $result[$articleId]['pseudoPrice'] = $sArticle['pseudoprice'];
            $result[$articleId]['articleUrl'] = $sArticle['linkDetailsRewrited'];
            $result[$articleId]['imageUrl'] = $sArticle['image']['src']['original'];
        }

        return $result;
    }

    private function xml2array($xml)
    {
        $arr = array();
        foreach ($xml as $element)
        {
            $tag = $element->getName();
            $e = get_object_vars($element);
            if (!empty($e)) {
                $arr[$tag] = $element instanceof SimpleXMLElement ? $this->xml2array($element) : $e;
            } else {
                $arr[$tag] = trim($element);
            }
        }

        return $arr;
    }

    /* end Navigation */

    /**
     * Exports the articles in CSV format
     *
     * @return void
     * @throws Exception
     */
    public function exportAction()
    {
        Shopware()->Plugins()->Controller()->ViewRenderer()->setNoRender();

        Shopware_Plugins_Frontend_FinSearchAPI_Bootstrap::checkLicense();

        $config = Shopware()->Plugins()->Frontend()->FinSearchAPI()->Config();

        $this->ar = \Shopware\Components\Api\Manager::getResource('article');

        $header_added = false;

        if ($this->Request()->getParam('lang')) {
            $this->languageCode = $this->Request()->getParam('lang');
            $builder = Shopware()->Models()->createQueryBuilder();
            $builder->select('locale.id')
                ->from('Shopware\Models\Shop\Locale', 'locale')
                ->where('locale.locale LIKE ?1')
                ->setParameter(1, $this->Request()->getParam('lang'));

            $result = $builder->getQuery()->getSingleScalarResult();

            $this->language_id = $result;

            if (empty($this->language_id)) {
                echo 'Sprachkürzel "' . $this->Request()->getParam('lang') . '" ist unbekannt.<br>';
                echo 'Exportiere "' . Shopware()->Shop()->getLocale()->getLocale() . '" stattdessen.<br>';
                $this->language_id = Shopware()->Shop()->getLocale()->getId();
            }
        } else {
            $this->languageCode = Shopware()->Shop()->getLocale()->getLocale();
            $this->language_id = Shopware()->Shop()->getLocale()->getId();
        }

        if ($this->Request()->getParam('start')) {
            $SqlLimitStart = $this->Request()->getParam('start');
        }

        if ($this->Request()->getParam('limit')) {
            $SqlLimit = $this->Request()->getParam('limit');
        } else {
            $SqlLimit = $config->exportlimit;
        }

        if (!$SqlLimitStart) {
            $SqlLimitStart = 0;
        }

        if (!$SqlLimit) {
            $SqlLimit = 100;
        }

        $categoryID = Shopware()->Shop()->get('parentID');

        $this->exportHelper = new \FinSearchAPI\Helpers\ExportHelper();
        $hideNoInStock = (int)$this->exportHelper->getHideNoInstock();

        if (empty($this->Request()->getParam('redirection')) || $this->exportHelper->checkIfProductStreamsTableIsEmpty()) {
            $this->exportHelper->importProductStreamsDataToDb();
        }

        if (!empty($hideNoInStock)) {
            $sql = "SELECT DISTINCT SQL_CALC_FOUND_ROWS articles.id
                FROM s_articles articles
                INNER JOIN s_articles_categories_ro ac
                ON ac.articleId = articles.id
                INNER JOIN s_articles_details details
                ON details.articleID = articles.id
                WHERE ac.categoryID = $categoryID 
                AND ( 
                        (articles.laststock = 0 AND details.instock >= 0) 
                        OR (articles.laststock = 1 AND details.instock > 0)
                    )
                ORDER BY articles.id
                LIMIT $SqlLimitStart, $SqlLimit";
        } else {
            $sql = "SELECT DISTINCT SQL_CALC_FOUND_ROWS articles.id
                FROM s_articles articles
                INNER JOIN s_articles_categories_ro ac
                ON ac.articleId = articles.id
                INNER JOIN s_articles_details details
                ON details.articleID = articles.id
                WHERE ac.categoryID = $categoryID
                ORDER BY articles.id
                LIMIT $SqlLimitStart, $SqlLimit";
        }

        $articles = Shopware()->Db()->fetchCol($sql);

        $sql = "SELECT FOUND_ROWS() as count";
        $countArticles = Shopware()->Db()->fetchOne($sql);

        if (count($articles) > 0) {
            $fp = $this->openFile($SqlLimitStart);

            foreach ($articles as $articleID) {
                $line = array();

                $article = $this->getArticle($articleID);

                if (!$article['id']) {
                    continue;
                }

                $translationReader = new \Shopware_Components_Translation();


                /* Übersetzung anhand der language_id auslesen */
                $translationLang = $translationReader->read($this->language_id, 'article', $article['id']);
                /* Übersetzung anhand der Shop-ID auslesen */
                $translationShop = $translationReader->read(Shopware()->Shop()->getId(), 'article', $article['id']);

                $translation = array_merge($translationLang, $translationShop);

                if (!empty($translation)) {
                    $article = array_merge($article, $translation);
                }

                $article['details'] = $this->sortArticleDetails($article['details'], $article['mainDetailId']);
//                var_dump($article);die;

                if ($config->exportdate == 1 && isset($article['details'][0]['releaseDate'])) {
                    $date = strtotime($article['details'][0]['releaseDate']->format('Y-m-d H:i:s'));
                } else {
                    $date = isset($article['added']) ? strtotime($article['added']->format('Y-m-d H:i:s')) : '';
                }

                $helper = new \FinSearchAPI\Components\Findologic\Helper();

                $line['id'] = $article['id'];
                $line['ordernumber'] = $this->cleanstring($this->getAllOrdernumbers($article));

                if (empty($line['ordernumber'])) {
                    continue;
                }
//                previous version of article name export
//                $line['name'] = $this->cleanstring($this->properlyEncodeString($article['name']));
                $line['name'] = $this->cleanstring(mb_convert_encoding(utf8_encode(html_entity_decode(utf8_decode($article['name']))), 'UTF-8'));
                $line['summary'] = $this->cleanstring($article['description']);
                $line['description'] = $this->cleanstring($article['descriptionLong']);
                $line['price'] = round(
                    $article['details'][0]['prices'][0]['price'] / 100 * (100 + $article['tax']['tax']), 2
                );
                $line['taxrate'] = round($article['tax']['tax']);
                $line['url'] = $this->Front()->Router()->assemble(array(
                    'sViewport' => 'detail',
                    'sArticle' => $article['id'],
                ));
                $line['image'] = $this->getExportImage($article);
                $line['attributes'] = $this->cleanstring($this->getAttributes($article));
                $line['keywords'] = $this->cleanstring($article['keywords']);
                $line['groups'] = Shopware()->Shop()->getId();
                $line['sales_frequency'] = (Shopware()->Db()->fetchOne(
                        'SELECT SUM(sales) FROM s_articles_details
                         WHERE articleID=' . $article['id']
                    ) + Shopware()->Db()->fetchOne(
                        'SELECT pseudosales FROM s_articles
                         WHERE id=' . $article['id']
                    ));
                $line['date_added'] = isset($date) ? $date : '';

                if ($SqlLimitStart == 0 && $header_added == false) {
                    $this->writeHeader($fp, $line);
                    $header_added = true;
                }

                // __debug($line);die;

                fputcsv($fp, $line, $this->delimiter, $this->enclosure);
            }

            fclose($fp);
        }

        if (($SqlLimitStart + $SqlLimit) < $countArticles) {
            $nextUrlArray = array(
                'controller' => 'FinSearchAPI',
                'action' => 'Export',
                'start' => ($SqlLimitStart + $SqlLimit),
                'limit' => $SqlLimit
            );

            if ($this->Request()->getParam('lang')) {
                $nextUrlArray['lang'] = $this->Request()->getParam('lang');
            }

            $nextUrl = Shopware()->Front()->Router()->assemble($nextUrlArray);

            if (method_exists($this->Request(), 'isSecure') && $this->Request()->isSecure()) {
                $nextUrl = str_replace('http://', 'https://', $nextUrl);
            }

            header('HTTP/1.1 302 Found');
            header('Location: ' . $nextUrl . '/redirection/1');

            echo 'SUCCESS: Unfinished';
        } else {
            header('HTTP/1.1 200 OK');
            echo 'SUCCESS: Finished<br /><br />';

            $this->exportHelper->truncateProductStreamsDataTable();

            if ($config->exportpath) {
                echo 'File: ' . $this->buildPathToExportFile() . $config->exportpath . '/findologic_' .
                    Shopware()->Shop()->getId() . '_' . strtolower($this->languageCode) . '.csv';
            } else {
                echo 'File: ' . $this->buildPathToExportFile() . 'findologic_' .
                    Shopware()->Shop()->getId() . '_' . strtolower($this->languageCode) . '.csv';
            }
        }

        exit(0);
    }

    /**
     * Gets the export thumbnail/original image
     *
     * @param array $article Article
     * @return string|void Image URL or void
     */
    protected function getExportImage($article) {
        if (isset($article['images'][0]['mediaId'])) {
            $mediaID = $article['images'][0]['mediaId'];

            // Find main image
            foreach ($article['images'] as $image) {
                if ($image['main'] == 1) {
                    $mediaID = $image['mediaId'];
                }
            }
        } else {
            return;
        }

        $config = Shopware()->Plugins()->Frontend()->FinSearchAPI()->Config();

        $url = Shopware()->Shop()->getAlwaysSecure() ? 'https://' : 'http://';
        $url .= Shopware()->Shop()->getHost() . Shopware()->Shop()->getBasePath() . '/';

        $mediaModel = Shopware()->Models()->find('Shopware\Models\Media\Media', $mediaID);

        if (!$mediaModel) {
            return;
        }

        $thumbnails = $mediaModel->getThumbnails();
        $thumbnailSizes = $mediaModel->getAlbum()->getSettings()->getThumbnailSize();
        $thumbnailSize = $config->thumbnailsize;

        if (is_numeric($thumbnailSize) && array_key_exists($thumbnailSizes[$thumbnailSize], $thumbnails)) {
            $imageUrl = $url . $thumbnails[$thumbnailSizes[$thumbnailSize]];
        } else {
            $imageUrl = $url . $mediaModel->getPath();
        }

        if (version_compare(Shopware::VERSION, '5.1.0', '>=')) {
            $mediaService = $this->get('shopware_media.media_service');
            $imageUrl = $mediaService->getUrl($imageUrl);
        }

        return $imageUrl;
    }

    /**
     * Builds the Path to the export file
     *
     * @return string
     */
    function buildPathToExportFile() {
        $request = $this->Request();

        $segments = array_filter(explode('/', $request->getBaseUrl()));
        $base = array_pop(array_filter(explode('/', $request->getBasePath())));

        while (!empty($segments)) {
            $segment = array_shift($segments);
            if ($segment == $base) {
                break;
            }
        }

        $baseUrl = '';
        for ($i = 0; $i < count($segments); $i++) {
            $baseUrl .= '/..';
        }

        return $baseUrl . '/';
    }

    /**
     * Gets article data from repository
     *
     * @param Int $articleID Article ID
     * @return array Article Data
     */
    protected static function getArticleOld($articleID) {

        $builder = Shopware()->Models()->createQueryBuilder();

        $builder->select(array(
            'article',
            'details',
            'tax',
            'images',
            'prices',
            'options',
            'propertyValues',
            'categories'
        ))->from('Shopware\Models\Article\Article', 'article')
            ->leftJoin('article.details', 'details')
            ->leftJoin('article.tax', 'tax')
            ->leftJoin('article.images', 'images')
            ->leftJoin('details.prices', 'prices')
            ->leftJoin('details.configuratorOptions', 'options')
            ->leftJoin('article.propertyValues', 'propertyValues')
            ->leftJoin('article.categories', 'categories', null, null, 'categories.id')
            ->where('article.id = ?1')
            ->andWhere('images.parentId IS NULL')
            ->andWhere('article.active = 1')
            ->setParameter(1, (int) $articleID);

        $query = $builder->getQuery();
        $result = $query->getOneOrNullResult(\Doctrine\ORM\AbstractQuery::HYDRATE_ARRAY);

        return $result;
    }

    /**
     * Gets article data from repository
     *
     * @param Int $article_id Article ID
     * @return array Article Data
     */
    public function getArticle($article_id)
    {
        $versionInt = (int) str_replace('.', '', Shopware()->Config()->version);

        $export_helper = new \FinSearchAPI\Helpers\ExportHelper();

        $product                    = $export_helper->getMainArticle($article_id);
        $product['details']         = $export_helper->getArticleDetails($article_id);
        $product['tax']             = $export_helper->getArticleTax($product['taxId'])[0];
        $product['images']          = $export_helper->getArticleImages($article_id);
        $export_helper->getArticleDetailsPrices($product);
        $export_helper->getArticleDetailsConfiguratiorOptions($product);
        $product['propertyValues']  = $export_helper->getArticlePropertyValues($article_id);

        if ($versionInt >= 510 ) {
            $product['categories'] = array_merge(
                $export_helper->getArticleCategories($article_id),
                $export_helper->getArticleProductStream($article_id)
            );
        } else {
            $product['categories'] = $export_helper->getArticleCategories($article_id);
        }

        return $product;
    }

    /**
     * Returns the translated article name and description
     *
     * @param array $article Article array
     * @return array translated Article array
     */
    protected function getArticleTranslation($article) {
        if (empty($article)) {
            return;
        }

        $shop_id = Shopware()->Shop()->getId();

        $sql = "SELECT fallback_id FROM s_core_shops WHERE id = $shop_id;";
        $fallback_id = Shopware()->Db()->fetchOne($sql);

        $translation = $this->ar->getOne($article['id']);

        if (!empty($translation['translations'][$shop_id])) {
            $article = array_merge($article, $translation['translations'][$shop_id]);
            unset($article['shopId']);

            return $article;
        }

        if (!empty($translation['translations'][$fallback_id])) {
            $article = array_merge($article, $translation['translations'][$fallback_id]);
            unset($article['shopId']);

            return $article;
        }

        return $article;
    }

    /**
     * Sorts the article details, so the main article is the first in the array
     *
     * @param array  $articleDetails Unsorted Article Details
     * @param string $mainDetailID   Main detail ID
     * @return array Sorted Article  Details
     */
    protected function sortArticleDetails($articleDetails, $mainDetailID) {
        $newArticleDetails = array();
        $index = 1;
        foreach ($articleDetails as $articleDetail) {
            if ($articleDetail['id'] == $mainDetailID) {
                $newArticleDetails[0] = $articleDetail;
            } else {
                $newArticleDetails[$index] = $articleDetail;
                $index++;
            }
        }

        foreach ($newArticleDetails as $key => $newArticleDetail) {
            if ($newArticleDetail['active'] == 0) {
                unset($newArticleDetails[$key]);
            }
        }

        ksort($newArticleDetails);

        return $newArticleDetails;
    }

    /**
     * Opens the file from server
     *
     * @param int $start If we need to delete file (0 = delete file)
     * @return resource File resource
     */
    protected function openFile($start) {
        $config = Shopware()->Plugins()->Frontend()->FinSearchAPI()->Config();

        if ($config->exportpath) {
            $file = $config->exportpath . '/findologic_' . Shopware()->Shop()->getId() . '_' . strtolower($this->languageCode) . '.csv';
        } else {
            $file = 'findologic_' . Shopware()->Shop()->getId() . '_' . strtolower($this->languageCode) . '.csv';
        }

        if ( ($start == 0) && file_exists($file) ) {
            unlink($file);
        }

        $fp = fopen($file, "a");

        if (!file_exists($file) || !is_readable($file) || !is_writable($file)) {
            die('Error - File can not be created/read/written');
        }

        return $fp;
    }

    /**
     * Writes the header to the csv
     *
     * @param resource $fp      File resource
     * @param array    $columns Columns
     */
    protected function writeHeader(&$fp, $columns) {
        $header = array();

        foreach ($columns as $column => $unused) {
            $header[] = $column;
        }

        $header = implode($this->delimiter, $header);

        fputs($fp, $header . "\n");
    }

    /**
     * Cleans string from tags, slashes and more
     *
     * @param string $val String to be cleaned
     * @return mixed|string Result string
     */
    protected function cleanString($val) {
        $val = strip_tags($val);
        $val = addslashes($val);
        $val = str_replace("\\", '', $val);
        $val = str_replace("\n", ' ', $val);
        $val = str_replace("\r", ' ', $val);
        $val = str_replace("\t", ' ', $val);

        return $val;
    }

    /**
     * Checks if a strings is already urlEncoded
     *
     * @param string $string String to be checked
     * @return bool          If string is urlencoded
     */
    private function is_urlEncoded($string) {
        $test_string = $string;

        while (urldecode($test_string) != $test_string) {
            $test_string = urldecode($test_string);
        }

        return urlencode($test_string) == $string;
    }

    /**
     * checks if a strings is already urlRawEncoded
     */
    private function is_urlRawEncoded($string) {
        $test_string = $string;
        while (rawurldecode($test_string) != $test_string) {
            $test_string = rawurldecode($test_string);
        }
        return rawurlencode($test_string) == $string;
    }

    /**
     * Gets all ordernumbers from the product
     *
     * @param array $article Product
     * @return string Order  numbers (ord1|ord2|ord3|...)
     */
    protected function getAllOrdernumbers($article) {
        $ordernumbers = array();

        foreach ($article['details'] as $articleDetail) {
            if ($articleDetail['inStock'] == 0 && $article['lastStock']) {
                continue;
            }

            $ordernumbers[] = $articleDetail['number'];

            if ($articleDetail['supplierNumber']) {
                $ordernumbers[] = $articleDetail['supplierNumber'];
            }

            if ($articleDetail['ean']) {
                $ordernumbers[] = $articleDetail['ean'];
            }
        }

        return implode("|", $ordernumbers);
    }

    /**
     * Gets all attributes from the product
     *
     * @param array $article Product
     * @return string        Attributes
     */
    protected function getAttributes($article) {

        /* Vendor image media path */
        $mediaPath = Shopware()->System()->sPathArticleImg . substr(Shopware()->Db()
                ->fetchOne('SELECT img FROM s_articles_supplier WHERE id=' . $article['supplierId']),12);

        if (empty($this->language_id)) {
            $this->language_id = 1;
        }

        $data = '';

        if ($article['supplierId']) {
            $supplierName = Shopware()->Db()->fetchOne(
                'SELECT 
                     name 
                 FROM
                     s_articles_supplier 
                 WHERE 
                     id=' . $article['supplierId']);
            $data .= 'vendor=' . urlencode(mb_convert_encoding(utf8_encode(html_entity_decode(utf8_decode($supplierName))), 'UTF-8'));
            $data .= '&ratings=' . urlencode(mb_convert_encoding(utf8_encode(html_entity_decode(utf8_decode($this->getArticleVotes($article)))), 'UTF-8'));
//            $data .= '&onsale=' . $this->getIsArticleOnSale($article);
            $data .= '&deliverytime=' . urlencode(mb_convert_encoding(utf8_encode(html_entity_decode(utf8_decode($this->getArticleDeliveryTime($article)))), 'UTF-8'));
//            $data .= '&stock=' . urlencode(mb_convert_encoding(utf8_encode(html_entity_decode(utf8_decode($this->getArticleInStockNumber($article)))), 'UTF-8'));
//            $data .= '&vendorimage=' . urlencode(mb_convert_encoding(utf8_encode(html_entity_decode(utf8_decode($mediaPath))), 'UTF-8'));
        }

        $this->buildCatPath($article['categories'], $data);


        $excludedDataArray = [];
        if (count($article['details']) > 0) {
            $this->buildConfiguratorAttributes($article['details'], $article['lastStock'], $data, $excludedDataArray);
        }

        if (count($article['propertyValues']) > 0) {
            $this->buildPropertiesAttributes($article['propertyValues'], $data, $excludedDataArray);
        }

        return $data;
    }

    /* Start add attributes */

    /**
     * Returns article votes
     * @param $article
     * @return float
     */
    public function getArticleVotes($article)
    {
        $votes = Shopware()->Models()->getRepository('Shopware\Models\Article\Vote')->createQueryBuilder('o')
            ->innerJoin('o.article', 'a')
            ->where('a.id = :id')->setParameter('id', $article['id'])
            ->getQuery()->getResult();

        $voteCount = 0;
        foreach ($votes as $vote) {
            $voteCount += $vote->getPoints();
        }

        return round($voteCount / count($votes), 1 );
    }

    /**
     * Returns is article on stock
     * @param $article
     * @return mixed
     */
    public function getArticleInStockNumber($article)
    {
        $details = $article['details'];

        foreach ($details as $item) {
            return $item['inStock'];
        }
    }

    /**
     * Returns article delivery time
     * @param array $article
     * @return string
     */
    public function getArticleDeliveryTime($article)
    {
        $details = $article['details'];

        foreach ($details as $item) {
            return empty($item['shippingTime']) ? false : $item['shippingTime'];
        }
    }

    /**
     * Returns is article on sale
     * @param array $article
     * @return int
     */
    public function getIsArticleOnSale($article)
    {
        return $article['lastStock'] ? 1 : 0;
    }

    /* End add attributes*/


    /**
     * Builds the category path for the product
     *
     * @param array $categories Categories list
     * @param array $data       Result data
     */
    protected function buildCatPath($categories, &$data) {
        if (!is_array($categories) || !count($categories) > 0) {
            return;
        }

        $shopCat = Shopware()->Db()->fetchOne(
            'SELECT category_id
             FROM s_core_shops
             WHERE id=' . Shopware()->Shop()->getId()
        );

        foreach ($categories as $category) {
            if ((int) $category['id'] == (int) 1) {
                continue;
            }

            $categoryID = $category['id'];
            $catArray = array();

            do {
                $ret = Shopware()->Db()->fetchRow(
                    "SELECT id, parent, description, active
                     FROM s_categories
                     WHERE id=$categoryID"
                );

                $categoryID = $ret['parent'];

                $catArray[] = array(
                    'id' => $ret['id'],
                    'active' => $ret['active'],
                    'description' => $this->properlyEncodeString($ret['description']),
                );

                if ($ret['id'] == $shopCat) {
                    break;
                }
            } while ($ret['parent'] != 1 && !empty($ret['parent']));

            if ($catArray[(count($catArray) - 1)]['id'] != $shopCat) {
                continue;
            }

            $catArray = array_reverse($catArray);

            unset($catArray[0]);

            if (count($catArray) == 0) {
                continue;
            }

            $catArrayDescription = array();

            foreach ($catArray as $catDescription) {
                if ($catDescription['active']) {
                    $catArrayDescription[] = $catDescription['description'];
                } else {
                    break;
                }
            }

            if (is_array($catArrayDescription) && count($catArrayDescription) > 0) {
                if ($data != '') {
                    $data .= '&';
                }

                $data .= 'cat=' . implode('_', $catArrayDescription);
            }
        }
    }

    /**
     * Builds all variant attributes from the product
     *
     * @param array $articleDetails Product variants
     * @param bool $lastStock
     * @param array $data Result data
     * @param array $excludedDataArray Data from variants with stock=0
     */
    protected function buildConfiguratorAttributes($articleDetails, $lastStock, &$data, &$excludedDataArray) {
        $shopId = Shopware()->Shop()->getId();
        $excludedData = array();
        $configuratorData = array();

        foreach ($articleDetails as $articleDetail) {
            if (is_array($articleDetail['configuratorOptions']) && count($articleDetail['configuratorOptions']) > 0) {
                foreach ($articleDetail['configuratorOptions'] as $configuratorOption) {
                    if ($shopId > 1) {
                        $sql = "SELECT objectdata FROM s_core_translations "
                            . "WHERE objecttype = 'configuratorgroup' "
                            . "AND objectkey = " . $configuratorOption['groupId'] . " "
                            . "AND objectlanguage = " . $shopId;

                        $configuratorOptionGroupName = unserialize(Shopware()->Db()->fetchOne($sql));

                        if (!empty($configuratorOptionGroupName['name'])) {
                            $configuratorOptionGroupName = $configuratorOptionGroupName['name'];
                        } else {
                            $sql = 'SELECT name FROM s_article_configurator_groups WHERE id='
                                . $configuratorOption['groupId'];
                            $configuratorOptionGroupName = Shopware()->Db()->fetchOne($sql);
                        }

                        $sql = "SELECT objectdata FROM s_core_translations "
                                . "WHERE objecttype = 'configuratoroption' "
                                . "AND objectkey = " . $configuratorOption['id'] . " "
                                . "AND objectlanguage = " . $shopId;

                        $configuratorOptionName = unserialize(Shopware()->Db()->fetchOne($sql));

                        $configuratorOptionName = $configuratorOptionName['name'];
                        $optionName = !empty($configuratorOptionName)
                            ? $configuratorOptionName
                            : $configuratorOption['name'];

                        if ($articleDetail['inStock'] == 0 && $lastStock) {
                            $excludedData[$configuratorOptionGroupName][] = $optionName;
                        } else {
                            $configuratorData[$configuratorOptionGroupName][] = $optionName;
                        }

                    } else {
                        $sql = 'SELECT name FROM s_article_configurator_groups WHERE id='
                            . $configuratorOption['groupId'];

                        $configuratorOptionName = Shopware()->Db()->fetchOne($sql);
                        $optionName = !empty($configuratorOptionName)
                            ? $configuratorOptionName
                            : $configuratorOption['name'];
                        if ($articleDetail['inStock'] == 0 && $lastStock) {
                            $excludedData[$configuratorOptionName][] = $optionName;
                        } else {
                            $configuratorData[$configuratorOptionName][] = $optionName;
                        }
                    }
                }
            }
        }

        foreach ($excludedData as $group => $values) {
            $values = array_unique($values);

            foreach ($values as $value) {
                $excludedDataArray[] = $this->properlyEncodeString($group) . '=' . $this->properlyEncodeString($value);
            }
        }

        foreach ($configuratorData as $group => $values) {
            $values = array_unique($values);

            foreach ($values as $value) {
                $data .= '&' . $this->properlyEncodeString($group) . '=' . $this->properlyEncodeString($value);
            }
        }
    }

    /**
     * Builds the article properties attributes
     *
     * @param array $propertyValues Properties values
     * @param array $data Result data
     * @param array $excludedDataArray
     */
    protected function buildPropertiesAttributes($propertyValues, &$data, $excludedDataArray) {
        $propertyValues = $this->getPropertiesAttributesTranslations($propertyValues);
        $propertyValues = array_filter($propertyValues, function($item) {
            return isset($item['value']);
        });

        $propertyValuesArray = [];
        foreach ($propertyValues as $propertyValue) {
            $propertyValuesArray[] = $this->properlyEncodeString($propertyValue['optionName']) . '='
                . $this->properlyEncodeString($propertyValue['value']);
        }

        foreach ($excludedDataArray as $excludedKey => $excludedValue) {
            $attributeValue = explode('=', $excludedValue);
            $attributeValue = $attributeValue[1];
            foreach ($propertyValuesArray as $propertyKey => $propertyValue) {
                $explodedPropertyValue = explode('=', $propertyValue);
                $explodedPropertyValue = $explodedPropertyValue[1];
                if ($excludedValue == $propertyValue || $explodedPropertyValue == $attributeValue) {
                    unset($propertyValuesArray[$propertyKey]);
                }
            }
        }

        foreach ($propertyValuesArray as $propertyValue) {
            $data .= '&' . $propertyValue;
        }
    }

    /**
     * Gets translations for properties attributes
     *
     * @param array $propertyValues Properties values
     * @return array Result array
     */
    private function getPropertiesAttributesTranslations($propertyValues) {
        $shopId = Shopware()->Shop()->getId();

        foreach ($propertyValues as $key => $value) {
            if ($shopId > 1) {
                $sql = "SELECT objectdata FROM s_core_translations "
                    . "WHERE objecttype = 'propertyoption' "
                    . "AND objectkey = " . $value['optionId'] . " "
                    . "AND objectlanguage = " . $shopId;

                $optionName = unserialize(Shopware()->Db()->fetchOne($sql));
                if (!empty($optionName['optionName'])) {
                    $propertyValues[$key]['optionName'] = $optionName['optionName'];
                } else {
                    $sql = 'SELECT name FROM s_filter_options WHERE id=' . $value['optionId'];
                    $propertyValues[$key]['optionName'] = Shopware()->Db()->fetchOne($sql);
                }

                $sql = "SELECT objectdata FROM s_core_translations "
                    . "WHERE objecttype = 'propertyvalue' "
                    . "AND objectkey = " . $value['id'] . " "
                    . "AND objectlanguage = " . $shopId;

                $optionValue = unserialize(Shopware()->Db()->fetchOne($sql));

                if (!empty($optionValue['optionValue'])) {
                    $propertyValues[$key]['value'] = $optionValue['optionValue'];
                }
            } else {
                $sql = 'SELECT name FROM s_filter_options WHERE id=' . $value['optionId'];
                $propertyValues[$key]['optionName'] = Shopware()->Db()->fetchOne($sql);
            }
        }

        return $propertyValues;
    }

    /**
     * Properly encodes given string
     *
     * @param string $string
     * @param string $encoding
     * @return string
     */
    protected function properlyEncodeString($string, $encoding = 'UTF-8')
    {
        return urlencode(mb_convert_encoding(utf8_encode(html_entity_decode(utf8_decode($string))), $encoding));
    }

    ##########################
    ###     Export End     ###
    ##########################

    ##########################
    ### Autocomplete Start ###
    ##########################

    /**
     * Fire the autocomplete query (echo result out)
     *
     * @return void
     * @throws Exception
     */
    public function autocompleteAction() {
        Shopware()->Plugins()->Controller()->ViewRenderer()->setNoRender();

        Shopware_Plugins_Frontend_FinSearchAPI_Bootstrap::checkLicense();

        if (!ini_get('allow_url_fopen')) {
            die('allow_url_fopen is not enabled, please check your server config');
        }

        $config = Shopware()->Plugins()->Frontend()->FinSearchAPI()->Config();
        $helper = new FinSearchAPIHelper();

        /*
         * submit all parameters to smart suggest proxy in addition
         * to standardized parameters for future-proofing
         */
        $parameters = $_GET;
        $parameters['shop'] = $helper->cleanShopkey($config->shopkey);
        $parameters['revision'] = Shopware_Plugins_Frontend_FinSearchAPI_Bootstrap::getVersion();
        $parameters['group'] = array(Shopware()->Shop()->getId());

        $url = $helper->cleanServiceurl($config->serviceurl) .
            'autocomplete.php?' . http_build_query($parameters, '', '&');

        $handle = fopen($url, 'r');

        if ($handle === false) {
            die('Could not connect to search service, please check your shop config');
        }

        $content = "";

        if ($handle) {
            while (!feof($handle)) {
                $content .= fread($handle, 512);
            }

            fclose($handle);
        }

        echo $content;
    }

    ##########################
    ###  Autocomplete End  ###
    ##########################

    ##########################
    ###    Search Start    ###
    ##########################

    /**
     * FINDOLOGIC search action. Renders search page
     *
     * @return void
     * @throws Exception
     */
    public function searchAction() {
        $config = Shopware()->Plugins()->Frontend()->FinSearchAPI()->Config();

        Shopware_Plugins_Frontend_FinSearchAPI_Bootstrap::checkLicense();
        $helper = new FinSearchAPIHelper();

        $data = new stdClass();
        $xml = '';
        $url = '';
        $params = '';
        $order = ($this->Request()->getParam('order')) ? $this->Request()->getParam('order') : 'none';
        $view = ($this->Request()->getParam('view')) ? $this->Request()->getParam('view') : 1;
        $count = ($this->Request()->getParam('count'))
            ? $this->Request()->getParam('count')
            : $helper->getFirstPageSize();
        $keywords = urlencode($this->Request()->getParam('sSearch'));
        $shopurl = Shopware()->Front()->Router()->assemble(array(
            'controller' => 'FinSearchAPI',
            'action' => 'search'
        ));
        $shopurl .= '?sSearch=' . $keywords;

        $helper->buildSearchUrl($url, $keywords, $view, $order, $count, $this->Request());
        $helper->setFilter($this->Request()->getParams(), $url, $data, $params);
        $this->callFindologic($url, $keywords, $xml);
        $this->checkForIdentifier($xml, $this->Request()->getParams());
        $this->checkForLandingpage($xml);
        $this->checkForPromotions($xml, $data);
        $helper->buildPagesizeDropdown($data, $count);
        $this->buildQueryData($xml, $data);
        $helper->buildFilters($xml, $data);
        $helper->buildProductData($xml, $data);
        $helper->buildOrderDropdown($data, $order);
        $helper->buildPaging($xml, $keywords, $params, $view, $data, $shopurl);

        $this->setColorTilePath($data);

        $tpl_data = array(
            'query' => $data->query,
            'request_url' => $url,
            'shopurl' => $shopurl,
            'order_dropdown' => $data->orderDropdown,
            'pagesize_dropdown' => $data->pagesizeDropdown,
            'selected_filter' => $data->selected_filter,
            'filter' => $data->filter,
            'promotion' => $data->promotion,
            'products' => $data->products,
            'products_count' => count($data->products),
            'url_params' => $params,
            'paging' => $data->paging,
            'language_locale' => Shopware()->Shop()->getLocale()->getLocale(),
            'displayBranding' => $config['displaybranding'],
        );

        $view = $this->View();
        $view->loadTemplate('plugins/fin_search_api/searchResults.tpl');
        $view->findologic = $tpl_data;
    }


    /**
     * Builds path for color tiles in filter
     *
     * @param $data
     * @return void
     */
    private function setColorTilePath( &$data ) {
        $colorName = !empty($data->filter['Color']['filter']) ? 'Color' : 'Farbe';
        $exportHelper = new \FinSearchAPI\Helpers\ExportHelper();
        $filters = $data->filter[$colorName]['filter'];

        foreach($filters as &$filterValue) {
            $media = $exportHelper->getMedia($filterValue['name']);
            $filterValue['image_path'] = $media[0]['media']['path'];
        }

        $data->filter[$colorName]['filter'] = $filters;
    }

    /**
     * Fires the FINDOLOGIC search
     *
     * @return mixed
     */
    protected function getFirstPageSize() {
        $config = Shopware()->Plugins()->Frontend()->FinSearchAPI()->Config();

        $tmpCount = $config->pagearticlescount;

        $tmpCount = explode(';', $tmpCount);

        if (!$tmpCount[0]) {
            $count = 16;
        } else {
            $count = $tmpCount[0];
        }

        return $count;
    }

    /**
     * build the search url for FINDOLOGIC
     */
    protected function BuildSearchURL(&$url, $keywords, $view, $order, $count) {
        $config = Shopware()->Plugins()->Frontend()->FinSearchAPI()->Config();

        if ($view > 1) {
            $first = ($view - 1) * $config->pagearticlescount;
        } else {
            $first = 0;
        }

        $url = $this->cleanServiceurl($config->serviceurl) .
            '?shopkey=' . $this->cleanShopkey($config->shopkey) .
            '&shopurl=' . Shopware()->Config()->basepath .
            '&userip=' . $this->getUserIP() .
            '&referer=' . urlencode($_SERVER['HTTP_REFERER']) .
            '&revision=' . Shopware_Plugins_Frontend_FinSearchAPI_Bootstrap::getVersion() .
            '&query=' . $keywords .
            '&group[]=' . Shopware()->Shop()->getId() .
            "&count=" . $count .
            "&first=" . $first;

        if ($order != "none") {
            $url .= '&order=' . urlencode($order);
        }

        if ($this->Request()->getParam('identifier')) {
            $url .= '&identifier=' . $this->Request()->getParam('identifier');
        }
    }

    /**
     * get the user ip Adress
     */
    protected function getUserIP() {
        $userIP = '';

        if (!isset($_SERVER['HTTP_X_FORWARDED_FOR'])) {
            $userIP = $_SERVER['REMOTE_ADDR'];
        } else {
            $clientList = explode(', ', $_SERVER['HTTP_X_FORWARDED_FOR']);
            $userIP = $clientList[0];
        }

        return $userIP;
    }

    /**
     * set the used filters
     */
    public function setFilter($get, &$url, &$data, &$params) {
        $currency = Shopware()->Shop()->getCurrency()->getCurrency();

        if ($val = $get ['catFilter']) {
            $tmpVal = $this->is_urlRawEncoded($val) ? $val : rawurlencode($val);
            $value = '&catFilter=' . $tmpVal;
            $data->selected_filter[] = array('group' => 'cat', "value" => $value, "name" => str_replace('_', ' > ', $val));
            $url .= '&attrib[cat][]=' . $tmpVal;
            $params .= $value;
        }

        if ($get['attrib']) {
            foreach ($get['attrib'] as $key => $arrayValues) {
                //if (true || $key != "price") {
                if (array_key_exists('min', $arrayValues) && array_key_exists('max', $arrayValues)) {
                    $tmpKey = $this->is_urlRawEncoded($key) ? $key : rawurlencode($key);
                    $value = "&attrib[" . $tmpKey . "][min]=" . $arrayValues['min'] . "&attrib[" . $tmpKey . "][max]=" . $arrayValues['max'];
                    $data->selected_filter[] = array(
                        'group' => $key,
                        'value' => $value,
                        'name' => (string) Shopware()->System()->sMODULES['sArticles']->sFormatPrice(round((float) $arrayValues['min'], 2)) . ' ' . $arrayValues['unit'] . ' - ' . Shopware()->System()->sMODULES['sArticles']->sFormatPrice(round((float) $arrayValues['max'], 2)) . ' ' . $arrayValues['unit']
                    );
                    $url .= $value;
                    $params .= $value;
                } else {
                    foreach ($arrayValues as $val) {
                        $tmpKey = $this->is_urlRawEncoded($key) ? $key : rawurlencode($key);
                        $tmpVal = $this->is_urlRawEncoded($val) ? $val : rawurlencode($val);
                        $value = "&attrib[" . $tmpKey . "][]=" . $tmpVal;
                        $data->selected_filter[] = array(
                            'group' => $key,
                            'value' => $value,
                            'name' => $val
                        );
                        $url .= $value;
                        $params .= $value;
                    }
                }
            }
        }

        foreach ($data->selected_filter as $key => $val) {
            //  Variante 2 - Scheint an sich erstmal zu funktionieren
            $data->selected_filter[$key]['params'] = str_replace($val['value'], "", $params);
        }
    }

    /**
     * Fires the FINDOLOGIC search
     *
     * @param string           $url      Search URL
     * @param string           $keywords Keywords
     * @param SimpleXMLElement $xml      Resulting XML object
     * @return mixed
     */
    protected function callFindologic(&$url, $keywords, &$xml) {
        $response = '';
        $info = array();

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_TIMEOUT, 3);
        $response = curl_exec($ch);
        $info = curl_getinfo($ch);
        curl_close($ch);

        if ($info['http_code'] != 200) {
            $redirectURL = $nextUrl = Shopware()->Front()->Router()->assemble(array('sViewport' => 'search'));

            return $this->redirect($redirectURL . '?sSearch=' . $keywords . '&fallback=1');
        } else {
            try {
                $xml = new SimpleXMLElement($response);
            } catch (Exception $e) {
                $redirectURL = $nextUrl = Shopware()->Front()->Router()->assemble(array('sViewport' => 'search'));

                return $this->redirect($redirectURL . '?sSearch=' . $keywords . '&fallback=1');
            }
        }
    }

    /**
     * Checks if identifier used (click an entry in auto complete)
     * and was only one article found, then redirect to the article
     *
     * @param SimpleXMLElement $xml Resulting XML object
     * @param array $params
     */
    protected function checkForIdentifier($xml, $params) {
        if ($params['identifier'] && $xml->results->count == 1) {
            $this->redirect(Shopware()->Front()->Router()->assemble(array(
                'module' => 'frontend',
                'controller' => 'detail',
                'sArticle' => $params['identifier'],
            )));
        }
    }

    /**
     * Checks if the result an landing page and redirect them in this case
     *
     * @param SimpleXMLElement $xml Resulting XML object
     */
    protected function checkForLandingpage($xml) {
        if ($xml->landingPage) {
            $this->redirect((string) $xml->landingPage['link']);
        }
    }

    /**
     * Check the result for promotions
     *
     * @param SimpleXMLElement $xml Resulting XML object
     * @param array $data           Result data
     * @return void
     */
    protected function checkForPromotions($xml, &$data) {
        if ($xml->promotion) {
            $data->promotion['image'] = (string) $xml->promotion['image'];
            $data->promotion['link'] = (string) $xml->promotion['link'];
        }
    }

    /**
     * build the pagesize dropdown
     */
    protected function buildPagesizeDropdown(&$data, $count) {
        $config = Shopware()->Plugins()->Frontend()->FinSearchAPI()->Config();

        $tmpCount = $config->pagearticlescount;

        $tmpCount = explode(';', $tmpCount);

        if (count($tmpCount) > 1) {
            foreach ($tmpCount as $countArray) {
                $data->pagesizeDropdown['sizes'][] = $countArray;
            }

            $data->pagesizeDropdown['selected'] = $count;
        }
    }

    /**
     * Builds the query data shown the user "for your search xxx were found yy products"
     *
     * @param SimpleXMLElement $xml Resulting XML object
     * @param array $data           Result data
     */
    protected function buildQueryData($xml, &$data) {
        $data->query['count'] = (int) $xml->results->count;

        $shopID = Shopware()->Shop()->getId();
        $localeID = Shopware()->Shop()->getLocale()->getId();

        $ifEmptyQueryString = empty($xml->query->queryString) ? '' : '&bdquo;' . $xml->query->queryString . '&ldquo;';
        switch ($xml->results->count) {
            case '0':
                $resultNothingText = Shopware()->Db()->fetchOne('
                    SELECT value FROM s_core_snippets
                    WHERE name="FinSearchAPIResultnothing"
                    AND shopID="' . $shopID . '"
                    AND localeID="' . $localeID . '"'
                );

                if (empty($resultNothingText)) {
                    $resultNothingText = Shopware()->Db()->fetchOne('
                        SELECT value FROM s_core_snippets
                        WHERE name="FinSearchAPIResultnothing"
                        AND localeID="' . $localeID . '"
                        AND value IS NOT NULL'
                    );
                }

                $data->query['string'] = sprintf($resultNothingText, $ifEmptyQueryString);
                break;

            case '1':
                $resultSingleText = Shopware()->Db()->fetchOne(
                    'SELECT value FROM s_core_snippets
                     WHERE name="FinSearchAPIResultsingle"
                     AND shopID="' . $shopID . '"
                     AND localeID="' . $localeID . '"'
                );

                if (empty($resultSingleText)) {
                    $resultSingleText = Shopware()->Db()->fetchOne(
                        'SELECT value FROM s_core_snippets
                         WHERE name="FinSearchAPIResultsingle"
                         AND localeID="' . $localeID . '"
                         AND value IS NOT NULL'
                    );
                }

                $data->query['string'] = sprintf($resultSingleText, $ifEmptyQueryString);
                break;

            default:
                $resultMultipleText = Shopware()->Db()->fetchOne(
                    'SELECT value FROM s_core_snippets
                     WHERE name="FinSearchAPIResultmultiple"
                     AND shopID="' . $shopID . '"
                     AND localeID="' . $localeID . '"'
                );

                if (empty($resultMultipleText)) {
                    $resultMultipleText = Shopware()->Db()->fetchOne(
                        'SELECT value FROM s_core_snippets
                         WHERE name="FinSearchAPIResultmultiple"
                         AND localeID="' . $localeID . '"
                         AND value IS NOT NULL'
                    );
                }

                $data->query['string'] = sprintf($resultMultipleText, $ifEmptyQueryString, $xml->results->count);
                break;
        }
    }

    /**
     * build the available filters
     */
    protected function buildFilters($xml, &$data) {

        foreach ($xml->filters->filter as $query_filter) {
            $newfilter = array();
            // Anzeigename
            if ($query_filter->display) {
                $newfilter ['display'] = (string) $query_filter->display;
            } else {
                $newfilter ['display'] = (string) $query_filter->name;
            }

            if ($query_filter->name == "cat") {

                if ($data->selected_filter) {
                    foreach ($data->selected_filter as $key => $selected) {
                        if ($selected['group'] == 'cat') {
                            $selectedFilter = str_replace('&catFilter=', '', $selected['value']);
                            break;
                        }
                    }
                }

                if ($selectedFilter) {
                    $selectedFilter = $this->is_urlRawEncoded($selectedFilter) ? rawurldecode($selectedFilter) : $selectedFilter;
                    $filterArray = explode('_', $selectedFilter);
                    $catFilter = $query_filter;

                    foreach ($filterArray as $selectedItem) {
                        foreach ($catFilter->items->item as $key => $catFiltersXml) {
                            if ($catFiltersXml->name == $selectedItem) {
                                $catFilter = $catFiltersXml;
                            }
                        }
                    }

                    foreach ($catFilter->items->item as $item) {
                        $filteritem = array();
                        $filteritem ['name'] = (string) $item->name;
                        $filteritem ['catFilter'] = (string) $selectedFilter . '_' . $item->name;
                        $filteritem ['selectedCatFilter'] = (string) $selectedFilter;
                        $filteritem ['frequency'] = (string) $item->frequency;
                        $filteritem ['type'] = (string) $query_filter->select;
                        $filteritem ['weight'] = (string) $item->weight;
                        $filteritem ['showFilterItem'] = $this->showFilterItemCheck($item->items->item);

                        $newfilter ['filter'][] = $filteritem;
                    }
                    $data->filter[(string) $query_filter->name] = $newfilter;

                    continue;
                }

                foreach ($query_filter->items->item as $item) {
                    $filteritem = array();
                    $filteritem ['name'] = (string) $item->name;
                    $filteritem ['frequency'] = (string) $item->frequency;
                    $filteritem ['type'] = (string) $query_filter->select;
                    $filteritem ['weight'] = (string) $item->weight;
                    $filteritem ['showFilterItem'] = $this->showFilterItemCheck($item->items->item);

                    $newfilter ['filter'][] = $filteritem;
                }

                $data->filter[(string) $query_filter->name] = $newfilter;
            } else {

                // nur bei Multiselect
                if ((string) $query_filter->select == 'multiselect') {
                    $newfilter ['selectedItems'] = (string) $query_filter->selectedItems;
                }
                // angabe des Types
                switch ($query_filter->type) {
                    case 'range-slider':
                        $newfilter ['type'] = (string) $query_filter->type;
                        $newfilter ['selectedmin'] = round((float) $query_filter->attributes->selectedRange->min, 2);
                        $newfilter ['selectedmax'] = round((float) $query_filter->attributes->selectedRange->max, 2);
                        $newfilter ['totalmin'] = round((float) $query_filter->attributes->totalRange->min, 2);
                        $newfilter ['totalmax'] = round((float) $query_filter->attributes->totalRange->max, 2);
                        $newfilter ['stepcount'] = floor((float) (floatval($query_filter->attributes->totalRange->max) - floatval($query_filter->attributes->totalRange->min)) / floatval($query_filter->attributes->stepSize));
                        $newfilter ['unit'] = (string) $query_filter->attributes->unit;
                        break;
                    case 'select':
                        $newfilter ['type'] = 'dropdown';
                        break;
                    default:
                        $newfilter ['type'] = (string) $query_filter->select;
                        break;
                }
                if ($query_filter->select == 'multiselect') {
                    $newfilter ['select'] = 'multiselect';
                }
                // not Slider
                if ($query_filter->type != "range-slider") {
                    foreach ($query_filter->items->item as $item) {
                        $filteritem = array();
                        if ($query_filter->name == "price") {
                            if ((string) $item->weight == 0) {
                                continue;
                            }
                            $currency = Shopware()->Shop()->getCurrency()->getCurrency();
                            $filteritem ['name'] = (string) $item->name;
                            $filteritem ['weight'] = (string) $item->weight;
                            $filteritem ['min'] = round((float) $item->parameters->min, 2);
                            $filteritem ['max'] = round((float) $item->parameters->max, 2);
                            $filteritem ['name'] = (string) Shopware()->System()->sMODULES['sArticles']->sFormatPrice(round((float) $item->parameters->min, 2)) . ' ' . $currency . ' - ' . Shopware()->System()->sMODULES['sArticles']->sFormatPrice(round((float) $item->parameters->max, 2)) . ' ' . $currency;
                        } else {
                            $filteritem ['name'] = (string) $item->name;
                            $filteritem ['frequency'] = (string) $item->frequency;
                            $filteritem ['weight'] = (string) $item->weight;
                            if (isset($item->selected)) { // bei multiselect
                                $filteritem ['selected'] = 'selected';
                            }
                        }
                        $newfilter ['filter'][] = $filteritem;
                    }
                }
                $data->filter[(string) $query_filter->name] = $newfilter;
            }
        }

        $shopID = Shopware()->Shop()->getId();
        $localeID = Shopware()->Shop()->getLocale()->getId();

        $cat = Shopware()->Db()->fetchOne('SELECT value FROM s_core_snippets WHERE name="FinSearchAPICatfilter" AND shopID="' . $shopID . '" AND localeID="' . $localeID . '"');
        if (empty($cat)) {
            $cat = Shopware()->Db()->fetchOne('SELECT value FROM s_core_snippets WHERE name="FinSearchAPICatfilter" AND localeID="' . $localeID . '" AND value IS NOT NULL');
        }
        $data->filter['cat']['display'] = $cat;

        $vendor = Shopware()->Db()->fetchOne('SELECT value FROM s_core_snippets WHERE name="FinSearchAPIVendorfilter" AND shopID="' . $shopID . '" AND localeID="' . $localeID . '"');
        if (empty($vendor)) {
            $vendor = Shopware()->Db()->fetchOne('SELECT value FROM s_core_snippets WHERE name="FinSearchAPIVendorfilter" AND localeID="' . $localeID . '" AND value IS NOT NULL');
        }
        $data->filter['vendor']['display'] = $vendor;

        $price = Shopware()->Db()->fetchOne('SELECT value FROM s_core_snippets WHERE name="FinSearchAPIPricefilter" AND shopID="' . $shopID . '" AND localeID="' . $localeID . '"');
        if (empty($price)) {
            $price = Shopware()->Db()->fetchOne('SELECT value FROM s_core_snippets WHERE name="FinSearchAPIPricefilter" AND localeID="' . $localeID . '" AND value IS NOT NULL');
        }
        $data->filter['price']['display'] = $price;
    }

    /**
     * Builds the params string over Ajax
     *
     * @return void
     */
    public function buildParamsStringAction() {
        $params = $this->Request()->getParam('params');
        $newparam = $this->Request()->getParam('newparam');
        parse_str($params, $queryparams);

        foreach ($newparam as $key => $value) {
            if ($key == 'cat') {
                $queryparams['catFilter'] = str_replace(' > ', '_', $value);
            } else {
                switch ($value['type']) {
                    case 'range':
                        $queryparams['attrib'][$key]['min'] = $value['min'];
                        $queryparams['attrib'][$key]['max'] = $value['max'];
                        $queryparams['attrib'][$key]['unit'] = $value['unit'];
                        break;
                    case 'multi':
                        unset($queryparams['attrib'][$key]);
                        foreach ($value['values'] as $ivalue) {
                            $queryparams['attrib'][$key] = $value['values'];
                        }
                        break;
                    default:
                        if ($key == 'price') {
                            $expl = explode('|', $value);
                            $queryparams['attrib'][$key]['min'] = $expl[0];
                            $queryparams['attrib'][$key]['max'] = $expl[1];
                        } else {
                            $queryparams['attrib'][$key][] = $value;
                        }
                        break;
                }
            }
        }

        if (!empty($queryparams['attrib']) || !empty($queryparams['catFilter'])) {
            $queryparams = $this->array_map_deep($queryparams, 'rawurlencode');

            $queryparams = http_build_query($queryparams);
            $queryparams = urldecode(preg_replace('/%5B[0-9]+%5D/simU', '%5B%5D', $queryparams));

            echo '&' . $queryparams;
        } else {
            echo '';
        }
        die;
    }

    /**
     * check if filter item show
     */
    protected function showFilterItemCheck($Item)
    {
        $showFilterItem = false;
        foreach ($Item as $subItem) {
            if (isset($subItem->items->item)) {
                foreach ($subItem->items->item as $subSubItem) {
                    if (isset($subSubItem->weight) && $subSubItem->weight > 0) {
                        $showFilterItem = true;
                        break;
                    }
                }
            } else {
                if ($subItem->weight > 0) {
                    $showFilterItem = true;
                    break;
                }
            }
            return $this->checkCategoryWeight($subItem);
        }

        return $showFilterItem;
    }

    /**
     * Check if category weight > 0 in category tree
     *
     * @param $category
     * @return bool
     */
    protected function checkCategoryWeight($category)
    {
        if (isset($category->items->item)) {

            if (!isset($category->items->item)) {
                return false;
            } else {

                foreach ($category->items->item as $subItem) {
                    if (isset($subItem->weight) && $subItem->weight > 0) {
                        return true;
                    } else {
                        return $this->checkCategoryWeight($subItem);
                    }
                }
            }
        }
    }

        /**
     * build the products data for the products listing
     */
    protected function buildProductData($xml, &$data) {
        $data->products = array();

        foreach ($xml->products->product as $product) {
            $article = Shopware()->Modules()->Articles()->sGetPromotionById('fix', 0, (int) $product['id']);

            if (!empty($article['articleID'])) {
                $data->products[] = $article;
            }
        }
    }

    /**
     * build the order dropdown
     */
    protected function buildOrderDropdown(&$data, $order) {
        $data->orderDropdown['selected'] = $order;
    }

    /**
     * build the paging nav
     */
    protected function buildPaging($xml, $keywords, $params, $view, &$data, $shopurl) {
        $data->paging['Page'] = $view;
        $data->paging['pages'] = ceil((int) $xml->results->count / (int) $xml->query->limit['count']);

        $data->paging['first'] = $shopurl . $params . '&order=' . $data->orderDropdown['selected'] . '&count=' . $data->pagesizeDropdown['selected'] . '&view=1';
        $data->paging['previous'] = $shopurl . $params . '&order=' . $data->orderDropdown['selected'] . '&count=' . $data->pagesizeDropdown['selected'] . '&view=' . ($view - 1);
        $data->paging['next'] = $shopurl . $params . '&order=' . $data->orderDropdown['selected'] . '&count=' . $data->pagesizeDropdown['selected'] . '&view=' . ($view + 1);
        $data->paging['last'] = $shopurl . $params . '&order=' . $data->orderDropdown['selected'] . '&count=' . $data->pagesizeDropdown['selected'] . '&view=' . $data->paging['pages'];
    }

    /**
     * Mapping array recursive
     *
     * @param array  $array    Array to be mapped
     * @param string $callback Callback function
     * @return array|mixed Callback function call result
     */
    private function array_map_deep($array, $callback) {
        $new = array();
        if (is_array($array))
            foreach ($array as $key => $val) {
                if (is_array($val)) {
                    $new[$key] = $this->array_map_deep($val, $callback);
                } else {
                    $new[$key] = call_user_func($callback, $val);
                }
            } else
            $new = call_user_func($callback, $array);
        return $new;
    }

    /**
     * cleaning shopkey only a-z,A-Z,0-9
     */
    private function cleanShopkey($key) {
        $key = preg_replace("/[^a-zA-Z0-9]/i", "", $key);
        return $key;
    }

    /**
     * cleaning serviceurl only a-z,A-Z,0-9,/,:,.
     */
    private function cleanServiceurl($key) {
        $key = preg_replace("/[^a-zA-Z0-9\/:._-]/i", "", $key);
        return $key;
    }

##########################
###     Search End     ###
##########################
}
