<?php
/** Class FinSearchAPIHelper */

/**
 * Class FinSearchAPIHelper contains helper functions for FINDOLOGIC plugin
 *
 * @author FINDOLOGIC GmbH
 * @copyright  Copyright (c) 2016
 */
class FinSearchAPIHelper
{
    /**
     * Gets the first page size
     *
     * @return int First page items count
     */
    public function getFirstPageSize()
    {
        $config = Shopware()->Plugins()->Frontend()->FinSearchAPI()->Config();
        $tmpCount = $config->pagearticlescount;
        $tmpCount = explode(';', $tmpCount);

        if (!$tmpCount[0]) {
            $count = 12;
        } else {
            $count = $tmpCount[0];
        }

        return $count;
    }

    /**
     * Builds the search url for FINDOLOGIC
     *
     * @param string $url
     * @param array  $keywords
     * @param object $view
     * @param string $order
     * @param int    $count
     * @param object $request
     * @return void
     */
    public function BuildSearchURL(&$url, $keywords, $view, $order, $count, $request)
    {
        $config = Shopware()->Plugins()->Frontend()->FinSearchAPI()->Config();

        if ($view > 1) {
            $first = ($view - 1) * $count;
        } else {
            $first = 0;
        }

        $url = $this->cleanServiceurl($config->serviceurl) .
            '?shopkey=' . $this->cleanShopkey($config->shopkey) .
            '&shopurl=' . Shopware()->Config()->basepath .
            '&userip=' . $this->getUserIP() .
            '&referer=' . urlencode($_SERVER['HTTP_REFERER']) .
            '&revision=' . Shopware_Plugins_Frontend_FinSearchAPI_Bootstrap::getVersion() .
            '&query=' . $keywords .
            '&group[]=' . Shopware()->Shop()->getId() .
            '&count=' . $count .
            '&first=' . $first;

        if ($order != 'none') {
            $url .= '&order=' . urlencode($order);
        }

        if ($request->getParam('identifier')) {
            $url .= '&identifier=' . $request->getParam('identifier');
        }
    }

    /**
     * Builds the category url for FINDOLOGIC
     *
     * @param string $url
     * @param string $order
     * @param int    $count
     * @param array  $attrib
     * @return void
     */
    public function buildCategoryUrl(&$url, $order, $count, $attrib)
    {
        $url = $url .
            // $this->buildAttributeString($attrib) .
            "&order=$order&count=$count";
    }

    /**
     * Cleans serviceurl only a-z,A-Z,0-9,/,:,.
     *
     * @param string $url
     * @return string Clean service URL
     */
    public function cleanServiceurl($url) {
        $url = preg_replace("/[^a-zA-Z0-9\/:._-]/i", '', $url);

        return $url;
    }

    /**
     * Cleans shopkey only a-z,A-Z,0-9
     *
     * @param string $key Shop key
     * @return string
     */
    public function cleanShopkey($key) {
        $key = preg_replace("/[^a-zA-Z0-9]/i", '', $key);

        return $key;
    }

    /**
     * Gets the user ip address
     *
     * @return string User IP address
     */
    protected function getUserIP()
    {
        if (!isset($_SERVER['HTTP_X_FORWARDED_FOR'])) {
            $userIP = $_SERVER['REMOTE_ADDR'];
        } else {
            $clientList = explode(', ', $_SERVER['HTTP_X_FORWARDED_FOR']);
            $userIP = $clientList[0];
        }

        return $userIP;
    }

    /**
     * Sets used filters
     *
     * @param array  $get    Request params
     * @param string $url    Page URL
     * @param object $data   Data
     * @param string $params URL params
     * @return void
     */
    public function setFilter($get, &$url, &$data, &$params)
    {
        if ($val = $get ['catFilter']) {
            $tmpVal = $this->is_urlRawEncoded($val) ? $val : rawurlencode($val);
            $value = '&catFilter=' . $tmpVal;
            $data->selected_filter[] = array(
                'group' => 'cat',
                'value' => $value,
                'name' => str_replace('_', ' > ', $val)
            );
            $url .= '&attrib[cat][]=' . $tmpVal;
            $params .= $value;
        }

        if ($get['attrib']) {
            foreach ($get['attrib'] as $key => $arrayValues) {
                //if (true || $key != "price") {
                if (array_key_exists('min', $arrayValues) && array_key_exists('max', $arrayValues)) {
                    $tmpKey = $this->is_urlRawEncoded($key) ? $key : rawurlencode($key);
                    $value = "&attrib[" . $tmpKey . "][min]=" . $arrayValues['min'] .
                        "&attrib[" . $tmpKey . "][max]=" . $arrayValues['max'];
                    $data->selected_filter[] = array(
                        'group' => $key,
                        'value' => $value,
                        'name' => (string) Shopware()->System()
                                    ->sMODULES['sArticles']
                                    ->sFormatPrice(round((float) $arrayValues['min'], 2))
                                    . ' ' . $arrayValues['unit'] . ' - ' .
                            Shopware()->System()
                                ->sMODULES['sArticles']
                                ->sFormatPrice(round((float) $arrayValues['max'], 2)) . ' ' . $arrayValues['unit']
                    );
                    $url .= $value;
                    $params .= $value;
                } else {
                    foreach ($arrayValues as $val) {
                        $tmpKey = $this->is_urlRawEncoded($key) ? $key : rawurlencode($key);
                        $tmpVal = $this->is_urlRawEncoded($val) ? $val : rawurlencode($val);
                        $value = '&attrib[' . $tmpKey . '][]=' . $tmpVal;
                        $data->selected_filter[] = array(
                            'group' => $key,
                            'value' => $value,
                            'name' => $val
                        );
                        $url .= $value;
                        $params .= $value;
                    }
                }
            }
        }

        foreach ($data->selected_filter as $key => $val) {
            //  Variante 2 - Scheint an sich erstmal zu funktionieren
            $data->selected_filter[$key]['params'] = str_replace($val['value'], '', $params);
        }
    }

    /**
     * Checks if a strings is already urlRawEncoded
     *
     * @param string $string String to check
     * @return string
     */
    public function is_urlRawEncoded($string)
    {
        $test_string = $string;

        while (rawurldecode($test_string) != $test_string) {
            $test_string = rawurldecode($test_string);
        }

        return rawurlencode($test_string) == $string;
    }

    /**
     * Builds the available filters
     *
     * @param SimpleXMLElement $xml
     * @param stdClass $data
     * @return void
     */
    public function buildFilters($xml, &$data)
    {
        foreach ($xml->filters->filter as $query_filter) {
            $newfilter = array();
            // Anzeigename
            if ($query_filter->display) {
                $newfilter ['display'] = (string) $query_filter->display;
            } else {
                $newfilter ['display'] = (string) $query_filter->name;
            }

            if ($query_filter->name == "cat") {

                if ($data->selected_filter) {
                    foreach ($data->selected_filter as $key => $selected) {
                        if ($selected['group'] == 'cat') {
                            $selectedFilter = str_replace('&catFilter=', '', $selected['value']);
                            break;
                        }
                    }
                }

                if ($selectedFilter) {
                    $selectedFilter = $this->is_urlRawEncoded($selectedFilter)
                        ? rawurldecode($selectedFilter)
                        : $selectedFilter;
                    $filterArray = explode('_', $selectedFilter);
                    $catFilter = $query_filter;

                    foreach ($filterArray as $selectedItem) {
                        foreach ($catFilter->items->item as $key => $catFiltersXml) {
                            if ($catFiltersXml->name == $selectedItem) {
                                $catFilter = $catFiltersXml;
                            }
                        }
                    }

                    foreach ($catFilter->items->item as $item) {
                        $filteritem = array();
                        $filteritem ['name'] = (string) $item->name;
                        $filteritem ['catFilter'] = (string) $selectedFilter . '_' . $item->name;
                        $filteritem ['selectedCatFilter'] = (string) $selectedFilter;
                        $filteritem ['frequency'] = (string) $item->frequency;
                        $filteritem ['type'] = (string) $query_filter->select;
                        $filteritem ['weight'] = (string) $item->weight;
                        $filteritem ['showFilterItem'] = $this->showFilterItemCheck($item->items->item);

                        $newfilter ['filter'][] = $filteritem;
                    }
                    $data->filter[(string) $query_filter->name] = $newfilter;

                    continue;
                }

                foreach ($query_filter->items->item as $item) {
                    $filteritem = array();
                    $filteritem ['name'] = (string) $item->name;
                    $filteritem ['frequency'] = (string) $item->frequency;
                    $filteritem ['type'] = (string) $query_filter->select;
                    $filteritem ['weight'] = (string) $item->weight;
                    $filteritem ['showFilterItem'] = $this->showFilterItemCheck($item->items->item);

                    $newfilter ['filter'][] = $filteritem;
                }

                $data->filter[(string) $query_filter->name] = $newfilter;
            }
            else {
                // nur bei Multiselect
                if ((string) $query_filter->select == 'multiselect') {
                    $newfilter ['selectedItems'] = (string) $query_filter->selectedItems;
                }
                // angabe des Types
                switch ($query_filter->type) {
                    case 'range-slider':
                        $newfilter ['type'] = (string) $query_filter->type;
                        $newfilter ['selectedmin'] = round((float) $query_filter->attributes->selectedRange->min, 2);
                        $newfilter ['selectedmax'] = round((float) $query_filter->attributes->selectedRange->max, 2);
                        $newfilter ['totalmin'] = round((float) $query_filter->attributes->totalRange->min, 2);
                        $newfilter ['totalmax'] = round((float) $query_filter->attributes->totalRange->max, 2);
                        $newfilter ['stepcount'] = floor((float) (floatval($query_filter->attributes->totalRange->max)
                                - floatval($query_filter->attributes->totalRange->min))
                                / floatval($query_filter->attributes->stepSize));
                        $newfilter ['unit'] = (string) $query_filter->attributes->unit;
                        break;
                    case 'select':
                        $newfilter ['type'] = 'dropdown';
                        break;
                    default:
                        $newfilter ['type'] = (string) $query_filter->select;
                        break;
                }
                if ($query_filter->select == 'multiselect') {
                    $newfilter ['select'] = 'multiselect';
                }
                // not Slider
                if ($query_filter->type != 'range-slider') {
                    foreach ($query_filter->items->item as $item) {
                        $filteritem = array();
                        if ($query_filter->name == 'price') {
                            if ((string) $item->weight == 0) {
                                continue;
                            }
                            $currency = Shopware()->Shop()->getCurrency()->getCurrency();
                            $filteritem ['name'] = (string) $item->name;
                            $filteritem ['weight'] = (string) $item->weight;
                            $filteritem ['min'] = round((float) $item->parameters->min, 2);
                            $filteritem ['max'] = round((float) $item->parameters->max, 2);
                            $filteritem ['name'] = (string) Shopware()->System()
                                    ->sMODULES['sArticles']
                                    ->sFormatPrice(round((float) $item->parameters->min, 2))
                                    . ' ' . $currency . ' - ' .
                                    Shopware()->System()
                                        ->sMODULES['sArticles']
                                        ->sFormatPrice(round((float) $item->parameters->max, 2)) . ' ' . $currency;
                        } else {
                            $filteritem ['name'] = (string) $item->name;
                            $filteritem ['frequency'] = (string) $item->frequency;
                            $filteritem ['weight'] = (string) $item->weight;
                            if (isset($item->selected)) { // bei multiselect
                                $filteritem ['selected'] = 'selected';
                            }
                            if ($query_filter->name == 'Color') {
                                if ($item->color) {
                                    $filteritem ['color'] = (string) $item->color;
                                }
                                if ($item->image) {
                                    $filteritem ['image'] = strstr(current(get_headers($item->image)), '200') ? (string) $item->image : false;
                                }

                            }
                        }
                        $newfilter ['filter'][] = $filteritem;
                    }
                }
                $data->filter[(string) $query_filter->name] = $newfilter;
            }
        }

        $shopID = Shopware()->Shop()->getId();
        $localeID = Shopware()->Shop()->getLocale()->getId();

        $cat = Shopware()->Db()->fetchOne(
            'SELECT value FROM s_core_snippets
             WHERE name="FinSearchAPICatfilter"
             AND shopID="' . $shopID . '"
             AND localeID="' . $localeID . '"'
        );
        if (empty($cat)) {
            $cat = Shopware()->Db()->fetchOne(
                'SELECT value FROM s_core_snippets
                 WHERE name="FinSearchAPICatfilter"
                 AND localeID="' . $localeID . '"
                 AND value IS NOT NULL');
        }
        // $data->filter['cat']['display'] = $cat;

        $vendor = Shopware()->Db()->fetchOne(
            'SELECT value FROM s_core_snippets
             WHERE name="FinSearchAPIVendorfilter"
             AND shopID="' . $shopID . '"
             AND localeID="' . $localeID . '"'
        );
        if (empty($vendor)) {
            $vendor = Shopware()->Db()->fetchOne(
                'SELECT value FROM s_core_snippets
                 WHERE name="FinSearchAPIVendorfilter"
                 AND localeID="' . $localeID . '"
                 AND value IS NOT NULL'
            );
        }
        $data->filter['vendor']['display'] = $vendor;

        $price = Shopware()->Db()->fetchOne(
            'SELECT value FROM s_core_snippets
             WHERE name="FinSearchAPIPricefilter"
             AND shopID="' . $shopID . '"
             AND localeID="' . $localeID . '"'
        );
        if (empty($price)) {
            $price = Shopware()->Db()->fetchOne(
                'SELECT value FROM s_core_snippets
                 WHERE name="FinSearchAPIPricefilter"
                 AND localeID="' . $localeID . '"
                 AND value IS NOT NULL'
            );
        }

        $data->filter['price']['display'] = $price;
    }

    /**
     * Checks if filter item show
     *
     * @param $Item
     * @return bool
     */
    protected function showFilterItemCheck($Item)
    {
        foreach ($Item as $subItem) {
            return $this->checkCategoryWeight($subItem);
        }
    }

    /**
     * Check if category weight > 0 in category tree
     *
     * @param $category
     * @return bool
     */
    protected function checkCategoryWeight($category)
    {
        if (!isset($category->items->item)) {
            return false;
        } else {
            foreach ($category->items->item as $subItem) {
                if (isset($subItem->weight) && $subItem->weight > 0) {
                    return true;
                } else {
                    return $this->checkCategoryWeight($subItem);
                }
            }
        }
    }

    /**
     * Creates attributes string
     *
     * @param array $attrib Attributes array
     * @return string
     */
    public function buildAttributeString($attrib)
    {
        $attributeString = '';
        foreach ($attrib as $key => $aValues) {
            foreach ($aValues as $k=>$v) {
                $encodedKey = rawurlencode($key);
                $encodedSubKey = rawurlencode($k);
                $encodedValue = rawurlencode($v);
                $attributeString .= "&attrib[$encodedKey][$encodedSubKey]=$encodedValue";
            }
        }

        return $attributeString;
    }

    /**
     * Calls FINDOLOGIC service for category page (getting filters for specific category)
     *
     * @param string $serviceBasicUrl Service basic URL
     * @param string $shopKey         Shop key
     * @param string $categoryName    Category name
     * @param array  $attrib          Attributes
     * @param int    $count           Products count
     * @param int    $order           Products order
     * @return SimpleXMLElement Response from FINDOLOGIC
     * @throws Exception
     */
    public function callFindologicForCategoryPage($serviceBasicUrl, $shopKey, $categoryName, $attrib, $count, $order, $view = 0)
    {
        $xml = '';
        try {
            $ctx = stream_context_create(array('http'=>
                array('timeout' => 3)
            ));

            if ($view > 1) {
                $first = ($view - 1) * $count;
            } else {
                $first = 0;
            }

            $serviceUrl = $serviceBasicUrl
                . '?shopkey=' . $shopKey
                . '&attrib[cat][]=' . $categoryName
                . $this->buildAttributeString($attrib)
                . "&count=$count"
                . "&order=" . urlencode($order)
                . "&first=$first";
            // __debug($serviceUrl);die;
            $string = file_get_contents($serviceUrl, false, $ctx);
            // __debug($string);die;

        } catch (Exception $e) {
            // Handle error here
            throw $e; // Throw it back
        }

        if (!empty($string)) {
            $xml = new SimpleXMLElement($string);
        }

        return $xml;
    }

    /**
     * Builds category name string (with sub-categories)
     *
     * @param object $request Request object
     * @return string Category string
     */
    public function buildCategoryName($request)
    {
        $categoryID = $request->getParams()['sCategory'];
        $category = Shopware()->Modules()->sCategories()->sGetCategoriesByParent($categoryID);
        $categoryNames = [];

        foreach ($category as $c) {
            $categoryNames[] = rawurlencode($c['name']);
        }

        $categoryNames = array_reverse($categoryNames);
        $categoryName = implode('_', $categoryNames);

        return $categoryName;
    }

    /**
     * Builds the pagesize dropdown
     *
     * @param array $data  Response data
     * @param int   $count Products count
     */
    public function buildPagesizeDropdown(&$data, $count) {
        $config = Shopware()->Plugins()->Frontend()->FinSearchAPI()->Config();

        $tmpCount = $config->pagearticlescount;

        $tmpCount = explode(';', $tmpCount);

        if (count($tmpCount) > 1) {
            foreach ($tmpCount as $countArray) {
                $data->pagesizeDropdown['sizes'][] = $countArray;
            }

            $data->pagesizeDropdown['selected'] = $count;
        }
    }

    /**
     * Builds the products data for the products listing
     *
     * @param SimpleXMLElement $xml  FINDOLOGIC data
     * @param array            $data Response data
     */
    public function buildProductData($xml, &$data) {
        $data->products = array();

        foreach ($xml->products->product as $product) {
            $article = Shopware()->Modules()->Articles()->sGetPromotionById('fix', 0, (int) $product['id']);

            if (!empty($article['articleID'])) {
                /*
                * Reordering of thumbnail images in order to show biggest image as main image
                * */
                $first = $article['image']['thumbnails'][0];
                $last = $article['image']['thumbnails'][2];
                $article['image']['thumbnails'][0] = $last;
                $article['image']['thumbnails'][2] = $first;

                $data->products[] = $article;
            }
        }

    }

    /**
     * Builds the order dropdown
     *
     * @param array  $data  Response data
     * @param string $order Order value
     */
    public function buildOrderDropdown(&$data, $order) {
        $data->orderDropdown['selected'] = $order;
    }

    /**
     * Builds the paging nav
     *
     * @param SimpleXMLElement $xml      FINDOLOGIC data
     * @param string           $keywords Key words
     * @param string           $params   Params
     * @param int              $view     View
     * @param array            $data     Response data
     * @param string           $shopurl  Shop URL
     */
    public function buildPaging($xml, $keywords, $params, $view, &$data, $shopurl) {
        $data->paging['Page'] = $view;
        $data->paging['pages'] = ceil((int) $xml->results->count / (int) $xml->query->limit['count']);

        $data->paging['first'] = $shopurl . $params . '&order=' . $data->orderDropdown['selected']
            . '&count=' . $data->pagesizeDropdown['selected'] . '&view=1';
        $data->paging['previous'] = $shopurl . $params . '&order=' . $data->orderDropdown['selected']
            . '&count=' . $data->pagesizeDropdown['selected'] . '&view=' . ($view - 1);
        $data->paging['next'] = $shopurl . $params . '&order=' . $data->orderDropdown['selected']
            . '&count=' . $data->pagesizeDropdown['selected'] . '&view=' . ($view + 1);
        $data->paging['last'] = $shopurl . $params . '&order=' . $data->orderDropdown['selected']
            . '&count=' . $data->pagesizeDropdown['selected'] . '&view=' . $data->paging['pages'];
    }

    /**
     * Get order param from request
     *
     * @param object $request Request object
     * @return string Order param value
     */
    public function getOrderParam($request)
    {
        $order = ($request->getParam('order')) ? $request->getParam('order') : "none";
        $order = ($order !== 'undefined') ? $order : "none";
        // __debug($order);die;

        return $order;
    }

    /**
     * Get count param from request
     *
     * @param object $request Request object
     * @return string Count param value
     */
    public function getCountParam($request)
    {
        $firstPageSize = $this->getFirstPageSize();
        $count = ($request->getParam('count')) ? $request->getParam('count') : $firstPageSize;

        return ($count !== 'undefined') ? $count : $firstPageSize;
    }

    /**
     * Before each request alive-test is performed with an timeout of 1 second, to test if FINDOLOGIC responding or not.
     *
     * @param int $timeout
     * @return string
     */
    public function aliveTest($timeout = 1)
    {
        $config = Shopware()->Plugins()->Frontend()->FinSearchAPI()->Config();
        $url = $this->cleanServiceurl($config->serviceurl) .
            'alivetest.php?shopkey=' . $this->cleanShopkey($config->shopkey);

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_TIMEOUT, $timeout);
        $responseAliveTest = curl_exec($ch);
        curl_close($ch);

        return $responseAliveTest == 'alive';
    }

}